SUBROUTINE SU_SURF_FLDS

!**** *SU_SURF_FLDS*  - Initialize definition of surface fields

!     Purpose.
!     --------
!       Initialize definition of surface fields (physics fields)

!       For each group of surface fields, we follow the following pattern:
!       - Define which field must be active (allocated):
!         This is the only location of this routine where tests on variables
!         like LLASSIMAROME, LMPHYS or other physics control keys are allowed.
!         Cases LECMWF and .NOT.LECMWF are treated separately.
!       - Compute number of fields allocated in this group.
!       - Allocate the group buffer, call to INI_SFLP2 or INI_SFLP3.
!       - Set-up attributes for active fields: call to SETUP_SFLP2 or SETUP_SFLP3.
!         Cases LECMWF and .NOT.LECMWF are treated separately.

!       We can notice that each group has an abbreviated name (example SB) and
!       a non abbreviated name (example SOILB). The abbreviated name appears
!       in the buffer name (example YSP_SB%YSB).

!       Prognostic groups appear in the following order: SB, SG, RR, CL, OM, EP, X2, CI.
!       Diagnostic groups appear in the following order: VF, VP, VV, VN, VH, VA, VG, VC,
!        V2, V3, VD, WS, WW, VX, XA, XR, X2, SFL, SFO

!**   Interface.
!     ----------
!        *CALL* SU_SURF_FLDS

!        Explicit arguments :
!        --------------------

!        Implicit arguments :
!        --------------------
!        MODULE SURFACE_FIELDS_MIX

!     Method.
!     -------
!        See documentation

!     Externals.
!     ----------
!        None

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud  *ECMWF*

!     Modifications.
!     --------------
!        Original : 01-Jul-2006
!          E. BAZILE : 26-Oct-2006 Corrections for LVGSN,LSOLV, etc ...
!          A. Alias  : 30-Nov-2006 Corrections to have several vertical layers in
!                                  deep SOIL (LTPROF)
!          R. El Khatib 09-Feb-2007 backfixing
!          A. Trojakova 27-Jun-2007 backfixing number of EXTRP
!          A. Alias  : 22-Mar-2007 Corrections Pour SURFEX 
!          JJMorcrette 20070223 DU, BC, OM, SU, VOL, SOaerosol climatological fields
!          J. Flemming   12-April-2008 GFL attribute extension for GEMS
!          Y. Seity : 27-Jan-2008 : modifications to run AROME without 
!                                   ALADIN-Isba fields LLASSIMAROME
!          R. El Khatib 17-Apr-2008 Fix a memory corruption
!          P. Lopez   : 26-Jun-2009 Added 100m wind components
!          P. Lopez   : 22-Oct-2008 Added ducting diagnostic fields
!          Y. Takaya : 29-Jan-2009  add ocean mixed layer fields
!          JJMorcrette: 20091201 Total and clear-sky direct SW radiation flux at surface 
!          H. Hersbach:04-Dec-2009 10-m neutral wind, friction velocity, log forecast z0h
!                                  Move Charnock from constant to time-dependent trajectory
!          S. Boussetta/G.Balsamo      May 2009   Added variable LAI field
!          R. Forbese  01-Mar-2010 Added TCRW,TCSW and added 2 to IVDIAG
!          JJMorcrette 20100212 PP of CBASE, 0DEGL and VISIH
!          T. Wilhelmsson : 25-Mar-2010 add 6 hourly min/max fields
!          A. Alias  : Aug 2009 adding ocean currents for MF (J.F. Gueremy)
!                               Move setup of Nudging mask from group SD_VA to SD_VF
!                               Reset LTPROF to .F. when LMSE
!          K. Yessad (Aug 2009): prune conf 912, externalise conf 911.
!          A. Alias  : Nov 2009 adding ocean current names and use of LCURR for MF (J.F. Gueremy)
!                               adding CNAME when empty (YSD_VH)
!          P. Marguinaud : Jun 2010 setup albedo and emissivity (YSD_VF%YEMISF,
!          YSD_VF%YALBF) when SURFEX is not used       
!          K. Yessad (Aug 2010): add and use variables saying if surf fields are active.
!          Y. Bouteloup : (Jan 2011) Store surface flux for EDKF and MUSC
!          A. Alias  : MAr 2011  add sunshine duration
!          Y. Seity  : April 2011 activation of SURFRESERV.NEIGE in AROME
!          P. Lopez  11-Aug-2009 Added accumulated precipitation for assimilation
!          G. Balsamo/S. Boussetta 17-Apr-2011 Added land carbon dioxide fields (IVDIAG+6)
!          H. Hersbach 01-Apr-2011 auxiliary diagnostic radiation fields
!          F. Vana   26-Jan-2012  Historic surface moisture for TOMs BC.
!      R. El Khatib : 01-Mar-2012 LFPOS => NFPOS
!          R. El Khatib 09-Mar-2012 Groups VCLIP, VCLIH, SFLUX and SFORCS always allocated
!                                  for safe bounds checkings later
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0B  , ONLY : LECMWF
USE YOMCT0   , ONLY : N3DINI, LFPART2, NCONF, LCANARI, NFPOS, LAROME, LSCREEN, LSFORCS,&
 & LIFSTRAJ, N6BINS, LR2D
USE YOMLUN   , ONLY : NULOUT, NULNAM
USE YOM_YGFL, ONLY : YGHG, NGHG, YGRG, NGRG, LCO2SFC, LAEROSFC, LGRGSFC, YTRAC, LSF6SFC,&
 & NTRAC, LCH4SFC, LFIRE 
USE YOEWCOU  , ONLY : LWCOU  ,LWCOU2W, LWCUR
USE YOMFPC   , ONLY : NFPCLI
USE YOMGRB    ,ONLY : &
 & NGRBZ    ,NGRBTCW  ,NGRBTCWV ,NGRBSTL1 ,          &
 & NGRBSWVL1 ,NGRBSD   ,NGRBLSP  ,NGRBCP   ,NGRBSF   ,&
 & NGRBBLD  ,NGRBSSHF ,NGRBSLHF ,NGRBCHNK ,NGRBMSL  ,&
 & NGRBSDOR ,NGRBISOR ,NGRBANOR ,NGRBSLOR ,NGRBTCC  ,&
 & NGRB10U  ,NGRB10V  ,NGRB2T   ,NGRB2D   ,NGRBSSRD ,&
 & NGRBSTL2 ,NGRBSWVL2 ,NGRBLSM  ,NGRBSR   ,NGRBAL   ,&
 & NGRBSTRD ,NGRBSSR  ,NGRBSTR  ,NGRBTSR  ,NGRBTTR  ,&
 & NGRBEWSS ,NGRBNSSS ,NGRBE    ,NGRBSTL3 ,NGRBSWVL3 ,&
 & NGRBCCC  ,NGRBLCC  ,NGRBMCC  ,NGRBHCC  ,NGRBLGWS ,&
 & NGRBMGWS ,NGRBGWD  ,NGRBSRC  ,NGRBMX2T ,NGRBMN2T ,&
 & NGRBMX2T6,NGRBMN2T6,NGRBRO   ,NGRBSRO  ,NGRBSSRO ,&  
 & NGRBTCO3 ,NGRBIEWS ,NGRBINSS ,&
 & NGRBISHF ,NGRBIE   ,NGRBLSRH ,NGRBSKT  ,NGRBSP   ,&
 & NGRBSTL4 ,NGRBSWVL4 ,NGRBCSF  ,NGRBLSF  ,NGRBFAL  ,&
 & NGRBFSR  ,NGRBFLSR ,NGRBVIMD ,NGRBPARCS,NGRBTISR,&
 & NGRBCBASE,NGRB0DEGL,NGRBVISIH                    ,&
 & NGRBBLH  ,NGRBSUND ,NGRBPAR ,NGRBUVB ,NGRBCAPE ,NGRBFDIR, NGRBCDIR,&
 & NGRBCVL  ,NGRBCVH  ,NGRBTVL  ,NGRBTVH  ,&
 & NGRBLAIL ,NGRBLAIH ,NGRBSLT ,&
 & NGRBISTL1,NGRBISTL2,NGRBISTL3,NGRBISTL4,&
 & NGRBASN  ,NGRBRSN  ,NGRBTSN  ,&
 & NGRBCI   ,NGRBSSTK  ,NGRBTCLW ,NGRBTCIW ,NGRBTCRW ,NGRBTCSW,&
 & NGRBTSRC ,NGRBTTRC ,NGRBSSRC ,NGRBSTRC ,&
 & NGRBES   ,NGRBSMLT ,NGRB10FG ,NGRB10FG6,NGRBLSPF ,&   
 & NGRBALUVP,NGRBALUVD,NGRBALNIP,NGRBALNID,&
 & NGRBAERDEP,NGRBAERLTS,NGRBAERSCC       ,&
 & NGRBAERLS01,NGRBAERLS02,NGRBAERLS10,NGRBAERLS03,NGRBAERLS04,NGRBAERLS11,&
 & NGRBAERLS05,NGRBAERLS06,NGRBAERLS12,NGRBAERLS07,NGRBAERLS08,NGRBAERLS09 ,&
 & NGRBCO2NBF ,NGRBCO2OF ,NGRBCO2APF ,NGRBCO2FIRE ,&
 & NGRBSDFOR,NGRBSF6APF,NGRBCH4F,NGRBCH4FIRE,NGRBUCUR, NGRBVCUR,&
 & NGRB100U, NGRB100V ,NGRB10NU ,NGRB10NV ,NGRBZUST,&
 & NGRBDNDZN, NGRBDNDZA, NGRBDCTB, NGRBTPLB, NGRBTPLT, NGRBSSS,&
 & NGRBNEE  , NGRBGPP ,NGRBREC  ,NGRBINEE ,NGRBIGPP ,NGRBIREC,&  !LandCarbon
 & NGRBOCT  ,NGRBOCS  ,NGRBOCU  ,NGRBOCV  ,NGRBOCVVS,&        !KPP
 & NGRBOCVDF,NGRBOCDEP,NGRBOCLDP,NGRBOCLZ ,NGRBADVT ,&        !KPP
 & NGRBADVS ,NGRBOCUC ,NGRBOCVC ,NGRBUSTRC,NGRBVSTRC          !KPP
USE YOMDPHY  , ONLY : NVXP     ,NVXP2    ,NCXP     , &
 & NVEXTR  ,NVEXTRRAD,NVXTR2   ,NCEXTR   ,NCOM    ,LTPROF  ,LDIRCLSMOD !KPP
USE TRAJECTORY_MOD, ONLY : NGP5, NTRAJ_CST
USE YOMPHY   , ONLY : LMPHYS   ,LNEIGE   ,LGWDC    ,LNEBCO   ,LAEROSEA ,&
 & LAEROLAN ,LAEROSOO ,LAERODES ,LAEROVOL,LAEROSUL ,LRELAXT  ,LRELAXW  ,&
 & LSNV     ,LSOLV    ,LFGEL    ,LNEBR   ,LVGSN    ,LO3ABC   ,CGMIXLEN ,&
 & LPHSPSH  ,LEDKF    ,LCOEFKTKE
USE YOMNUD   , ONLY : LNDRIV
USE YOMARPHY , ONLY : LMSE
USE YOMMSE   , ONLY : NSURFEXCTL, NSURFEXCTLMAX
USE YOMMCC   , ONLY : LCURR, LMCC04
USE YOEPHY   , ONLY : LEPHYS, LE4ALB, LECURR, LEOCML, LESMOSS
USE SURFACE_FIELDS_MIX, ONLY : JPMAXSFLDS, NOFFTRAJ, NOFFTRAJ_CST,NSTRAJGRIB,&
 & NPROGSURF, NPROGSURFL, NSURF, NSURFL, NDIMSURF, NDIMSURFL,&
 & INI_SFLP3, SETUP_SFLP3, INI_SFLP2, SETUP_SFLP2, &
 & YSP_SBD, YSP_SB, YSP_ACT_SB, YSP_SGD, YSP_SG, YSP_ACT_SG, &
 & YSP_RRD, YSP_RR, YSP_ACT_RR, YSP_CLD, YSP_CL, YSP_ACT_CL, &
 & YSP_CID, YSP_CI, YSP_ACT_CI, YSP_EPD, YSP_EP, YSP_X2D, YSP_X2, YSP_ACT_X2, &
 & YSD_VFD, YSD_VF, YSD_ACT_VF, YSD_VPD, YSD_VP, YSD_ACT_VP, &
 & YSD_VVD, YSD_VV, YSD_ACT_VV, YSD_VND, YSD_VN, YSD_ACT_VN, &
 & YSD_VHD, YSD_VH, YSD_ACT_VH, YSD_VAD, YSD_VA, YSD_ACT_VA, &
 & YSD_VCD, YSD_VC, YSD_ACT_VC, YSD_V2D, YSD_V2, YSD_ACT_V2, &
 & YSD_VDD, YSD_VD, YSD_ACT_VD, YSD_WSD, YSD_WS, YSD_ACT_WS, &
 & YSD_WWD, YSD_WW, YSD_ACT_WW, YSD_VXD, YSD_VX, YSD_ACT_VX, &
 & YSP_OMD, YSP_OM, YSP_ACT_OM, YSD_V3D, YSD_V3, YSD_ACT_V3, &
 & YSD_XAD, YSD_XA, YSD_XRD, YSD_XR, YSD_X2D, YSD_X2, &
 & YSD_SFLD, YSD_SFL,YSD_SFOD, YSD_SFO
USE YOMPHYDS, ONLY : CEXTRP, NEXTRPGB, EXTRPDF, CXTRP2, NXTRP2GB, XTRP2DF,&
 & CVEXTRA, NVEXTRAGB, VEXTRADF, CVEXRAD, NVEXRADGB, VEXRADDF, CVEXTR2, NVEXTR2GB, VEXTR2DF,&
 & NSFORC, NSFLUX
USE YOPHLC   , ONLY : LSPHLC
USE YOMOBS   , ONLY : LVDFTRAJ
USE YOMDIM   , ONLY : NFLEVG

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM) :: IVCLIN,IVO3ABC,IVCAN,IVWV,IVWW,IVSF,IVCLIH,IVDIAG,&
 & IVCLIX,IVCLIA,IVCLIP,IVCLIV,ICSS,IVSO,IVSG,IVRS,ICLS, ICOM,IVOM,IVO2,IVO3          !KPP  
INTEGER(KIND=JPIM) :: IGRBCODES(100)
INTEGER(KIND=JPIM) :: JGHG, JGRG, JTRAC,JFLD, JBIN
INTEGER(KIND=JPIM) :: IREQIN,IREQINDF,IREQINCUR,ITRAJ,ICEXTR
INTEGER(KIND=JPIM) :: INOGRIB=-999
INTEGER(KIND=JPIM) :: IINDXTRP2
CHARACTER(LEN=16)  :: CLNAMES(100)
CHARACTER(LEN=16)  :: CLBLANK='                '
CHARACTER(LEN=16)  :: CLSFORC='                '
CHARACTER(LEN=16)  :: CLSFLUX='                '
REAL(KIND=JPRB)    :: ZDEFAULT(100),ZTDFLT
LOGICAL            :: LLNOSURF, LLCURRENT
LOGICAL            :: LLFP_CLASSIC, LLFP_SURFEX
LOGICAL            :: LL_CALL_VDIFLCZ ! VDIFLCZ is called under CPG_DYN

LOGICAL :: LL_FIRST_CALL=.TRUE.
LOGICAL :: LLASSIMAROME      ! T if LAROME except if AROME Assim (F)

REAL(KIND=JPRB)    :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "posnam.intfb.h"

!     ------------------------------------------------------------------

#include "namphyds.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('SU_SURF_FLDS',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*      0.  PRELIMINARY SETTINGS, AND NAMELIST READING.
!           -------------------------------------------

LLNOSURF = LR2D
LLASSIMAROME = LAROME.AND.(.NOT.(LSCREEN .OR. (NCONF == 131)))
LLCURRENT= LECMWF .AND. LECURR .AND. (.NOT. LMCC04)
IF (LLCURRENT) THEN
  IREQINCUR=1
ELSE
  IREQINCUR=-1
ENDIF
LL_CALL_VDIFLCZ=(.NOT.(LMPHYS.OR.LEPHYS)).AND.LSPHLC
! Define additive surface fields to read when fullpos is active
LLFP_CLASSIC = (NFPOS /= 0).AND.(NFPCLI == 3)
LLFP_SURFEX  = (NFPOS /= 0).AND..NOT.LFPART2.AND.LMSE

! ky: reset of LTPROF to F must probably be done elsewhere.
IF (LMPHYS.AND.(.NOT.LLASSIMAROME)) THEN
  IF(LMSE) THEN
    LTPROF=.FALSE.
    WRITE(NULOUT,*) ' INFO ---- LTPROF reset to .F. when using surfex '
  ENDIF
ENDIF

NOFFTRAJ = 0
NOFFTRAJ_CST = 0
NPROGSURF = 0
NPROGSURFL = 0
NSURF = 0
NSURFL = 0
NDIMSURF = 0
NDIMSURFL = 0

IF(N3DINI == 1) THEN
  IREQINDF=-1
  ZTDFLT = 250._JPRB
ELSE
  IREQINDF=1
  ZTDFLT = 0._JPRB
ENDIF

! Defaults for extra fields

CEXTRP  (:,:) = CLBLANK
NEXTRPGB(:,:) = INOGRIB
EXTRPDF (:,:) = 0.0_JPRB

CXTRP2  (:)   = CLBLANK
NXTRP2GB(:)   = INOGRIB
XTRP2DF (:)   = 0.0_JPRB

CVEXTRA  (:)  = CLBLANK
NVEXTRAGB(:)  = INOGRIB
VEXTRADF (:)  = 0.0_JPRB

CVEXRAD  (:)  = CLBLANK
NVEXRADGB(:)  = INOGRIB
VEXRADDF (:)  = 0.0_JPRB

CVEXTR2  (:)  = CLBLANK
NVEXTR2GB(:)  = INOGRIB
VEXTR2DF (:)  = 0.0_JPRB


! This is a very strange use of extra fields /MH
IF (LCANARI) THEN
  ! remark: 'PRWA' = field nr 1 + field nr 2; 'PRSN' = field nr 3 + field nr 4
  CXTRP2(1) = 'SURFPREC.EAU.CON'
  CXTRP2(2) = 'SURFPREC.EAU.GEC'
  CXTRP2(3) = 'SURFPREC.NEI.CON'
  CXTRP2(4) = 'SURFPREC.NEI.GEC' 
  IINDXTRP2 = 5
  IF(LSOLV) THEN
    CXTRP2(5) = 'ATMONEBUL.BASSE '
    CXTRP2(6) = 'SURFXFLU.MEVAP.E'
    CXTRP2(7) = 'SURFXEVAPOTRANSP'
    IINDXTRP2 = 8
  ENDIF
  CXTRP2(IINDXTRP2+0) = 'SURFFLU.LAT.MEVA'
  CXTRP2(IINDXTRP2+1) = 'SURFACCPLUIE'
  CXTRP2(IINDXTRP2+2) = 'SURFACCNEIGE'
  CXTRP2(IINDXTRP2+3) = 'SURFACCGRAUPEL'
ENDIF

IF(LECMWF) THEN
  NSFLUX = 0
  NSFORC = 0
ELSE
  NSFLUX = 2
  NSFORC = 2
ENDIF

!    READ NAMELIST.

CALL POSNAM(NULNAM,'NAMPHYDS')
READ(NULNAM,NAMPHYDS)

!     ------------------------------------------------------------------
!*      1.  SETUP SURFACE PROGNOSTIC VAR.
!           -----------------------------


! * Group SB=SOILB

! - define which field must be active:
IF (LECMWF) THEN
  YSP_ACT_SB%L_T=(.NOT.LLNOSURF)
  YSP_ACT_SB%L_Q=(.NOT.LLNOSURF)
  YSP_ACT_SB%L_TL=(.NOT.LLNOSURF)
ELSE
  YSP_ACT_SB%L_T=(.NOT.LLNOSURF).AND.LMPHYS.AND.((LSOLV.AND.LTPROF).OR.(.NOT.LLASSIMAROME))
  YSP_ACT_SB%L_Q=(.NOT.LLNOSURF).AND.LMPHYS.AND.((LSOLV.AND.LTPROF).OR.(.NOT.LLASSIMAROME))
  YSP_ACT_SB%L_TL=(.NOT.LLNOSURF).AND.LMPHYS.AND.((LSOLV.AND.LTPROF).OR.(.NOT.LLASSIMAROME)) &
   & .AND.((LSOLV.AND.LFGEL).OR.LMSE)
ENDIF

! - compute number of fields allocated in this group:
IVSO=0
IF (YSP_ACT_SB%L_T) IVSO=IVSO+1
IF (YSP_ACT_SB%L_Q) IVSO=IVSO+1
IF (YSP_ACT_SB%L_TL) IVSO=IVSO+1
IF (IVSO>0) THEN
  ! ICSS = number of reservoirs
  IF (LECMWF) THEN
    ICSS=4
  ELSE
    IF (LTPROF) THEN
      ICSS=3
    ELSE
      ICSS=1
    ENDIF
  ENDIF
ELSE
  ICSS=0
ENDIF

! - call to INI_SFLP3, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_SB%YSB)
IF(ASSOCIATED(YSP_SB%YSB)) DEALLOCATE(YSP_SB%YSB)
ALLOCATE(YSP_SB%YSB(JPMAXSFLDS))
CALL INI_SFLP3(YSP_SBD,YSP_SB%YSB,IVSO,ICSS,.TRUE.,'SOILB - SP_SB   ')
YSP_SB%YT  => YSP_SB%YSB(JPMAXSFLDS)
YSP_SB%YQ  => YSP_SB%YSB(JPMAXSFLDS)
YSP_SB%YTL => YSP_SB%YSB(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ZDEFAULT(1:ICSS)=ZTDFLT
  IF(YSP_ACT_SB%L_T) THEN
    IGRBCODES(1:ICSS) = (/NGRBSTL1,NGRBSTL2,NGRBSTL3,NGRBSTL4/)
    CLNAMES  (1:ICSS) = (/'LEV1TEMPERATURE','LEV2TEMPERATURE', &
     & 'LEV3TEMPERATURE','LEV4TEMPERATURE'/)
    YSP_SB%YT => YSP_SB%YSB(YSP_SBD%IPTR)
    CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YT,KGRIB=IGRBCODES(1:ICSS),&
     & CDNAME=CLNAMES(1:ICSS),PDEFAULT=ZDEFAULT(1:ICSS),KTRAJ=1,&
     & KREQIN=SPREAD(IREQINDF,1,4))
  ENDIF
  IF(YSP_ACT_SB%L_Q) THEN
    IGRBCODES(1:ICSS) = (/NGRBSWVL1,NGRBSWVL2,NGRBSWVL3,NGRBSWVL4/)
    CLNAMES  (1:ICSS) = (/'LEV1RESERV.EAU  ','LEV2RESERV.EAU  ', &
     & 'LEV3RESERV.EAU  ','LEV4RESERV.EAU  '/)
    YSP_SB%YQ => YSP_SB%YSB(YSP_SBD%IPTR)
    CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YQ,KGRIB=IGRBCODES(1:ICSS),&
    & CDNAME=CLNAMES(1:ICSS),PDEFAULT=ZDEFAULT(1:ICSS),KTRAJ=1,&
    & KREQIN=SPREAD(IREQINDF,1,4))
  ENDIF
  IF(YSP_ACT_SB%L_TL) THEN
    IGRBCODES(1:ICSS) = (/NGRBISTL1,NGRBISTL2,NGRBISTL3,NGRBISTL4/)
    CLNAMES  (1:ICSS) = (/'LEV1ICETEMP  ','LEV2ICETEMP  ', &
     & 'LEV3ICETEMP  ','LEV4ICETEMP  '/)
    YSP_SB%YTL => YSP_SB%YSB(YSP_SBD%IPTR)
    CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YTL,KGRIB=IGRBCODES(1:ICSS),&
     & CDNAME=CLNAMES(1:ICSS),PDEFAULT=ZDEFAULT(1:ICSS),&
     & KREQIN=SPREAD(IREQINDF,1,4))
  ENDIF
ELSE
  ZDEFAULT(1:ICSS)=ZTDFLT
  IF(YSP_ACT_SB%L_T) THEN
    YSP_SB%YT => YSP_SB%YSB(YSP_SBD%IPTR)
    IF (ICSS == 3) THEN
      CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YT,KGRIB=(/NGRBSTL2,NGRBSTL3,NGRBSTL4/),&
       & CDNAME=(/'PROFTEMPERATURE ','LEV3TEMPERATURE ','LEV4TEMPERATURE '/),&
       & PDEFAULT=ZDEFAULT(1:ICSS),KTRAJ=0,KREQIN=SPREAD(1,1,3))
    ELSEIF (ICSS == 1) THEN
      CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YT,KGRIB=(/NGRBSTL2/),&
       & CDNAME=(/'PROFTEMPERATURE '/),PDEFAULT=ZDEFAULT(1:ICSS), KTRAJ=1)
    ENDIF
  ENDIF
  IF(YSP_ACT_SB%L_Q) THEN
    YSP_SB%YQ => YSP_SB%YSB(YSP_SBD%IPTR)
    IF (ICSS == 3) THEN
      CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YQ,KGRIB=(/NGRBSWVL2,NGRBSWVL3,NGRBSWVL4/),&
       & CDNAME=(/'PROFRESERV.EAU  ','LEV3RESERV.EAU  ','LEV4RESERV.EAU  '/), &
       & PDEFAULT=ZDEFAULT(1:ICSS), KTRAJ=0,KREQIN=SPREAD(1,1,3))
    ELSEIF (ICSS == 1) THEN
      CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YQ,KGRIB=(/NGRBSWVL2/),CDNAME=(/'PROFRESERV.EAU  '/),&
       & KTRAJ=1)
    ENDIF
  ENDIF
  IF(YSP_ACT_SB%L_TL) THEN
    YSP_SB%YTL => YSP_SB%YSB(YSP_SBD%IPTR)
    IF (ICSS == 3) THEN
      CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YTL, &
       & CDNAME=(/'PROFRESERV.GLACE','LEV3RESERV.GLACE','LEV4RESERV.GLACE'/),&
       & PDEFAULT=ZDEFAULT(1:ICSS), KTRAJ=0,KREQIN=SPREAD(1,1,3))
    ELSEIF (ICSS == 1) THEN
      CALL SETUP_SFLP3(YSP_SBD,YSP_SB%YTL,CDNAME=(/'PROFRESERV.GLACE'/), KTRAJ=1)
    ENDIF
  ENDIF
ENDIF


! * Group SG=SNOWG

! - define which field must be active:
IF (LECMWF) THEN
  YSP_ACT_SG%L_F=.TRUE.
  YSP_ACT_SG%L_A=.TRUE.
  YSP_ACT_SG%L_R=.TRUE.
  YSP_ACT_SG%L_T=.TRUE.
ELSE
  YSP_ACT_SG%L_F=LMPHYS
  YSP_ACT_SG%L_A=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV.AND.(LSNV.OR.LVGSN)
  YSP_ACT_SG%L_R=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV.AND.(LSNV.OR.LVGSN)
  YSP_ACT_SG%L_T=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV.AND.LVGSN
ENDIF

! - compute number of fields allocated in this group:
IVSG=0
IF (YSP_ACT_SG%L_F) IVSG=IVSG+1
IF (YSP_ACT_SG%L_A) IVSG=IVSG+1
IF (YSP_ACT_SG%L_R) IVSG=IVSG+1
IF (YSP_ACT_SG%L_T) IVSG=IVSG+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_SG%YSG)
IF(ASSOCIATED(YSP_SG%YSG)) DEALLOCATE(YSP_SG%YSG)
ALLOCATE(YSP_SG%YSG(JPMAXSFLDS))
CALL INI_SFLP2(YSP_SGD,YSP_SG%YSG,IVSG,.TRUE.,'SNOWG - SP_SG   ')
YSP_SG%YF => YSP_SG%YSG(JPMAXSFLDS)
YSP_SG%YA => YSP_SG%YSG(JPMAXSFLDS)
YSP_SG%YR => YSP_SG%YSG(JPMAXSFLDS)
YSP_SG%YT => YSP_SG%YSG(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSP_ACT_SG%L_F) THEN
    YSP_SG%YF => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YF,PDEFAULT=ZTDFLT,&
     &KGRIB=NGRBSD ,CDNAME='SURFRESERV.NEIGE',KTRAJ=1,KREQIN=IREQINDF)
  ENDIF
  IF (YSP_ACT_SG%L_A) THEN
    YSP_SG%YA => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YA,PDEFAULT=ZTDFLT,&
     & KGRIB=NGRBASN,CDNAME='SNOW ALBEDO     ',KTRAJ=1,KREQIN=IREQINDF)
  ENDIF
  IF (YSP_ACT_SG%L_R) THEN
    YSP_SG%YR => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YR,PDEFAULT=ZTDFLT,&
     & KGRIB=NGRBRSN,CDNAME='SNOW DENSITY    ',KTRAJ=1,KREQIN=IREQINDF)
  ENDIF
  IF (YSP_ACT_SG%L_T) THEN
    YSP_SG%YT => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YT,PDEFAULT=ZTDFLT,&
     & KGRIB=NGRBTSN,CDNAME='SNOW TEMPERATURE',KTRAJ=1,KREQIN=IREQINDF)
  ENDIF
ELSE
  IF (YSP_ACT_SG%L_F) THEN
    YSP_SG%YF => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YF,KGRIB=NGRBSD ,CDNAME='SURFRESERV.NEIGE',KTRAJ=1,KREQIN=1)
  ENDIF
  IF (YSP_ACT_SG%L_A) THEN
    YSP_SG%YA => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YA,KGRIB=NGRBASN, CDNAME='SURFALBEDO NEIGE', KTRAJ=1,KREQIN=1)
  ENDIF
  IF (YSP_ACT_SG%L_R) THEN
    YSP_SG%YR => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YR,KGRIB=NGRBRSN, CDNAME='SURFDENSIT.NEIGE',KTRAJ=1,KREQIN=1)
  ENDIF
  IF (YSP_ACT_SG%L_T) THEN
    YSP_SG%YT => YSP_SG%YSG(YSP_SGD%IPTR)
    CALL SETUP_SFLP2(YSP_SGD,YSP_SG%YT,CDNAME='SURFALBEDO HISTO')
  ENDIF
ENDIF


! * Group RR=RESVR

! - define which field must be active:
IF (LECMWF) THEN
  YSP_ACT_RR%L_T=.TRUE.
  YSP_ACT_RR%L_W=.TRUE.
  YSP_ACT_RR%L_FC=.FALSE.
  YSP_ACT_RR%L_IC=.FALSE.
  YSP_ACT_RR%L_FP1=LFPART2.AND.(LMPHYS.OR.LEPHYS)
ELSE
  YSP_ACT_RR%L_T=LMPHYS
  YSP_ACT_RR%L_W=LMPHYS.AND.(.NOT.LLASSIMAROME)
  YSP_ACT_RR%L_FC=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV
  YSP_ACT_RR%L_IC=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.((LSOLV.AND.LFGEL).OR.LMSE)
  YSP_ACT_RR%L_FP1=LFPART2.AND.(LMPHYS.OR.LEPHYS)
ENDIF

! - compute number of fields allocated in this group:
IVRS=0
IF (YSP_ACT_RR%L_T) IVRS=IVRS+1
IF (YSP_ACT_RR%L_W) IVRS=IVRS+1
IF (YSP_ACT_RR%L_FC) IVRS=IVRS+1
IF (YSP_ACT_RR%L_IC) IVRS=IVRS+1
IF (YSP_ACT_RR%L_FP1) IVRS=IVRS+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_RR%YRR)
IF(ASSOCIATED(YSP_RR%YRR)) DEALLOCATE(YSP_RR%YRR)
ALLOCATE(YSP_RR%YRR(JPMAXSFLDS))
CALL INI_SFLP2(YSP_RRD,YSP_RR%YRR,IVRS,.TRUE.,'RESVR - SP_RR   ')
YSP_RR%YT => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YW => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YFC => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YIC => YSP_RR%YRR(JPMAXSFLDS)
YSP_RR%YFP1 => YSP_RR%YRR(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSP_ACT_RR%L_T) THEN
    YSP_RR%YT => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YT,PDEFAULT=ZTDFLT,&
     & KGRIB=NGRBSKT ,CDNAME='SKINTEMPERATURE ',KTRAJ=1,KREQIN=IREQINDF)
  ENDIF
  IF (YSP_ACT_RR%L_W) THEN
    YSP_RR%YW => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YW,PDEFAULT=ZTDFLT,&
     & KGRIB=NGRBSRC ,CDNAME='SKINRESERV.EAU  ',KTRAJ=1,KREQIN=IREQINDF)
  ENDIF
  ! FC and IC currently not active.
ELSE
  IF (YSP_ACT_RR%L_T) THEN
    YSP_RR%YT => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YT,KGRIB=NGRBSTL1,CDNAME='SURFTEMPERATURE ',KTRAJ=1, KREQIN=1)
  ENDIF
  IF (YSP_ACT_RR%L_W) THEN
    YSP_RR%YW => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YW,KGRIB=NGRBSWVL1,CDNAME='SURFRESERV.EAU  ',KTRAJ=1, KREQIN=1)
  ENDIF
  IF (YSP_ACT_RR%L_FC) THEN
    YSP_RR%YFC => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YFC,KGRIB=NGRBSRC,CDNAME='SURFRESERV.INTERC', KTRAJ=1, KREQIN=1)
  ENDIF
  IF (YSP_ACT_RR%L_IC) THEN
    YSP_RR%YIC => YSP_RR%YRR(YSP_RRD%IPTR)
    CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YIC,CDNAME='SURFRESERV.GLACE', KTRAJ=1, KREQIN=1)
  ENDIF
ENDIF
IF (YSP_ACT_RR%L_FP1) THEN
  YSP_RR%YFP1 => YSP_RR%YRR(YSP_RRD%IPTR)
  CALL SETUP_SFLP2(YSP_RRD,YSP_RR%YFP1,KGRIB=NGRBSTL1 ,CDNAME='INTSURFTEMPERATURE')
ENDIF  

  
! * Group CL=CLS

! - define which field must be active:
IF(LECMWF) THEN
  YSP_ACT_CL%L_TCLS=.FALSE.
  YSP_ACT_CL%L_HUCLS=.FALSE.
ELSE
  YSP_ACT_CL%L_TCLS=LMPHYS.AND.LDIRCLSMOD
  YSP_ACT_CL%L_HUCLS=LMPHYS.AND.LDIRCLSMOD
ENDIF

! - compute number of fields allocated in this group:
ICLS=0
IF (YSP_ACT_CL%L_TCLS) ICLS=ICLS+1
IF (YSP_ACT_CL%L_HUCLS) ICLS=ICLS+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_CL%YCL)
IF(ASSOCIATED(YSP_CL%YCL)) DEALLOCATE(YSP_CL%YCL)
ALLOCATE(YSP_CL%YCL(JPMAXSFLDS))
CALL INI_SFLP2(YSP_CLD,YSP_CL%YCL,ICLS,.TRUE.,'CLS   - SP_CL   ')
YSP_CL%YTCLS => YSP_CL%YCL(JPMAXSFLDS)
YSP_CL%YHUCLS => YSP_CL%YCL(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active.
ELSE
  IF (YSP_ACT_CL%L_TCLS) THEN
    YSP_CL%YTCLS => YSP_CL%YCL(YSP_CLD%IPTR)
    CALL SETUP_SFLP2(YSP_CLD,YSP_CL%YTCLS,KGRIB=NGRBSTL1,CDNAME='CLSTEMPERATURE  ',KTRAJ=1, KREQIN=1)
  ENDIF
  IF (YSP_ACT_CL%L_HUCLS) THEN
    YSP_CL%YHUCLS => YSP_CL%YCL(YSP_CLD%IPTR)
    CALL SETUP_SFLP2(YSP_CLD,YSP_CL%YHUCLS,KGRIB=NGRBSTL1,CDNAME='CLSHUMI.RELATIVE',KTRAJ=1, KREQIN=1)
  ENDIF
ENDIF


! * Group OM=OML (KPP)

! - define which field must be active:
IF (LECMWF) THEN
  YSP_ACT_OM%L_TO=LEOCML
  YSP_ACT_OM%L_SO=LEOCML
  YSP_ACT_OM%L_UO=LEOCML
  YSP_ACT_OM%L_VO=LEOCML
ELSE
  YSP_ACT_OM%L_TO=.FALSE.
  YSP_ACT_OM%L_SO=.FALSE.
  YSP_ACT_OM%L_UO=.FALSE.
  YSP_ACT_OM%L_VO=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVOM=0
IF (YSP_ACT_OM%L_TO) IVOM=IVOM+1
IF (YSP_ACT_OM%L_SO) IVOM=IVOM+1
IF (YSP_ACT_OM%L_UO) IVOM=IVOM+1
IF (YSP_ACT_OM%L_VO) IVOM=IVOM+1
IF (IVOM>0) THEN
  ICOM=NCOM
ELSE
  ICOM=0
ENDIF

! - call to INI_SFLP3, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_OM%YOM)
IF(ASSOCIATED(YSP_OM%YOM)) DEALLOCATE(YSP_OM%YOM)
ALLOCATE(YSP_OM%YOM(JPMAXSFLDS))
CALL INI_SFLP3(YSP_OMD,YSP_OM%YOM,IVOM,ICOM,.TRUE.,'OML   - SP_OM   ')
YSP_OM%YTO  => YSP_OM%YOM(JPMAXSFLDS)
YSP_OM%YSO  => YSP_OM%YOM(JPMAXSFLDS)
YSP_OM%YUO  => YSP_OM%YOM(JPMAXSFLDS)
YSP_OM%YVO  => YSP_OM%YOM(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN

  ! For ocean mixed layer model, NREQIN=2 was introduced to read
  ! initial condition in SUGRIBO.F90, not in SUGRIBG.F90 
  ! NREQIN = -1 : not read I.C.
  !        =  1 : read ICGG????INIT file (KFILE=0)
  !        =  2 : read ICGG????INIOC file

  IF (YSP_ACT_OM%L_TO) THEN
    IGRBCODES(1:ICOM) = NGRBOCT  
    CLNAMES  (1:ICOM) = 'OCEAN TEMP     '
    YSP_OM%YTO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YTO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM)) 
  ENDIF
  IF (YSP_ACT_OM%L_SO) THEN
    IGRBCODES(1:ICOM) = NGRBOCS  
    CLNAMES  (1:ICOM) = 'OCEAN SAL      '
    YSP_OM%YSO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YSO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))
  ENDIF
  IF (YSP_ACT_OM%L_UO) THEN
    IGRBCODES(1:ICOM) = NGRBOCU  
    CLNAMES  (1:ICOM) = 'OCEAN VEL U    '
    YSP_OM%YUO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YUO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))
  ENDIF
  IF (YSP_ACT_OM%L_VO) THEN
    IGRBCODES(1:ICOM) = NGRBOCV  
    CLNAMES  (1:ICOM) = 'OCEAN VEL V    '
    YSP_OM%YVO => YSP_OM%YOM(YSP_OMD%IPTR)
    CALL SETUP_SFLP3(YSP_OMD,YSP_OM%YVO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))
  ENDIF
ELSE
  ! no field currently active.
ENDIF


! * Group EP=EXTRP

IF (LL_FIRST_CALL) NULLIFY(YSP_EP%YEP)
IF(ASSOCIATED(YSP_EP%YEP)) DEALLOCATE(YSP_EP%YEP)
ALLOCATE(YSP_EP%YEP(JPMAXSFLDS))
CALL INI_SFLP3(YSP_EPD,YSP_EP%YEP,NVXP,NCXP,.TRUE.,'EXTRP - SP_EP   ')

IF (YSP_EPD%NUMFLDS > 0) THEN
  DO JFLD=1,YSP_EPD%NUMFLDS
    CALL SETUP_SFLP3(YSP_EPD,YSP_EP%YEP(JFLD),&
     & KGRIB=NEXTRPGB(1:YSP_EPD%NLEVS,JFLD),CDNAME=CEXTRP(1:YSP_EPD%NLEVS,JFLD),&
     & PDEFAULT=EXTRPDF(1:YSP_EPD%NLEVS,JFLD))
  ENDDO
ENDIF
    

! * Group X2=XTRP2

! - define which field must be active:
IF(LECMWF) THEN
  YSP_ACT_X2%L_X2(1:JPMAXSFLDS)=.FALSE.
ELSE
  YSP_ACT_X2%L_X2(1:4)=LCANARI
  IINDXTRP2 = 5
  IF (LSOLV) THEN
    YSP_ACT_X2%L_X2(5:7) = .TRUE.
    IINDXTRP2 = 8
  ENDIF
  YSP_ACT_X2%L_X2(IINDXTRP2+0:IINDXTRP2+3)=.TRUE.
  YSP_ACT_X2%L_X2(IINDXTRP2+4:JPMAXSFLDS)=.FALSE.
ENDIF

! - compute number of fields allocated in this group: assumed to be equal to NVXP2.

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_X2%YX2)
IF(ASSOCIATED(YSP_X2%YX2)) DEALLOCATE(YSP_X2%YX2)
ALLOCATE(YSP_X2%YX2(JPMAXSFLDS))
CALL INI_SFLP2(YSP_X2D,YSP_X2%YX2,NVXP2,.TRUE.,'XTRP2 - SP_X2   ')

! - set-up attributes for active fields:
IF (YSP_X2D%NUMFLDS > 0) THEN
  DO JFLD=1,YSP_X2D%NUMFLDS
    CALL SETUP_SFLP2(YSP_X2D,YSP_X2%YX2(JFLD),&
     &KGRIB=NXTRP2GB(JFLD),CDNAME=CXTRP2(JFLD),PDEFAULT=XTRP2DF(JFLD))
  ENDDO
ENDIF


! * Group CI=CANRI

! - define which field must be active:
IF(LECMWF) THEN
  YSP_ACT_CI%L_CI(1:JPMAXSFLDS)=.FALSE.
ELSE
  YSP_ACT_CI%L_CI(1:8)=LCANARI
  YSP_ACT_CI%L_CI(9:13)=LCANARI.AND.LSOLV
  YSP_ACT_CI%L_CI(14:JPMAXSFLDS)=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVCAN=0
DO JFLD=1,JPMAXSFLDS
  IF (YSP_ACT_CI%L_CI(JFLD)) IVCAN=IVCAN+1
ENDDO

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSP_CI%YCI)
IF(ASSOCIATED(YSP_CI%YCI)) DEALLOCATE(YSP_CI%YCI)
ALLOCATE(YSP_CI%YCI(JPMAXSFLDS))
CALL INI_SFLP2(YSP_CID,YSP_CI%YCI,IVCAN,.TRUE.,'CANRI - SP_CI   ')

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active.
ELSE
  IF (YSP_ACT_CI%L_CI( 1)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(1),CDNAME='SURFETP.GEOPOTEN')
  IF (YSP_ACT_CI%L_CI( 2)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(2),CDNAME='SURFETA.GEOPOTEN')
  IF (YSP_ACT_CI%L_CI( 3)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(3),CDNAME='SURFANALYSISMASK')
  IF (YSP_ACT_CI%L_CI( 4)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(4),CDNAME='CLSTEMPERATURE  ')
  IF (YSP_ACT_CI%L_CI( 5)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(5),CDNAME='CLSHUMI.RELATIVE')
  IF (YSP_ACT_CI%L_CI( 6)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(6),CDNAME='CLSVENT.ZONAL   ')
  IF (YSP_ACT_CI%L_CI( 7)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(7),CDNAME='CLSVENT.MERIDIEN')
  IF (YSP_ACT_CI%L_CI( 8)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(8),CDNAME='SURFSST.CLIM.   ')
  IF (YSP_ACT_CI%L_CI( 9)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(9) ,CDNAME='PROFINC.RESERV.1')
  IF (YSP_ACT_CI%L_CI(10)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(10),CDNAME='PROFINC.RESERV.2')
  IF (YSP_ACT_CI%L_CI(11)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(11),CDNAME='PROFINC.RESERV.3')
  IF (YSP_ACT_CI%L_CI(12)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(12),CDNAME='SURFINC.TEMPERAT')
  IF (YSP_ACT_CI%L_CI(13)) CALL SETUP_SFLP2(YSP_CID,YSP_CI%YCI(13),CDNAME='SURFINC.HUMIDITE')
ENDIF

!-------------------------------------------------------------------------

!*      2.  SETUP SURFACE DIAGNOSTIC VAR.
!           -----------------------------


! * Group VF=VARSF

! - define which field must be active:
!   (remark: LAN never used)
IF(LECMWF) THEN
  YSD_ACT_VF%L_Z0F=.TRUE.
  YSD_ACT_VF%L_ALBF=.TRUE.
  YSD_ACT_VF%L_EMISF=.TRUE.
  YSD_ACT_VF%L_GETRL=.TRUE.
  YSD_ACT_VF%L_LSM=.TRUE.
  YSD_ACT_VF%L_VEG=.FALSE.
  YSD_ACT_VF%L_VRLAN=.TRUE.
  YSD_ACT_VF%L_VRLDI=.TRUE.
  YSD_ACT_VF%L_SIG=.TRUE.
  YSD_ACT_VF%L_ALBSF=.FALSE.
  YSD_ACT_VF%L_LAN=.FALSE.
  YSD_ACT_VF%L_SST=.TRUE.
  YSD_ACT_VF%L_LZ0H=.TRUE.
  YSD_ACT_VF%L_CVL=.TRUE.
  YSD_ACT_VF%L_CVH=.TRUE.
  YSD_ACT_VF%L_TVL=.TRUE.
  YSD_ACT_VF%L_TVH=.TRUE.
  YSD_ACT_VF%L_LAIL=.TRUE.
  YSD_ACT_VF%L_LAIH=.TRUE.
  YSD_ACT_VF%L_SOTY=.TRUE.
  YSD_ACT_VF%L_CI=.TRUE.
  YSD_ACT_VF%L_UCUR=.TRUE.
  YSD_ACT_VF%L_VCUR=.TRUE.
  YSD_ACT_VF%L_Z0RLF=.FALSE.
  YSD_ACT_VF%L_SDFOR=.TRUE.
  YSD_ACT_VF%L_ALUVP=LE4ALB
  YSD_ACT_VF%L_ALUVD=LE4ALB
  YSD_ACT_VF%L_ALNIP=LE4ALB
  YSD_ACT_VF%L_ALNID=LE4ALB
  YSD_ACT_VF%L_AERDEP=LAEROSFC
  YSD_ACT_VF%L_AERLTS=LAEROSFC
  YSD_ACT_VF%L_AERSCC=LAEROSFC
  YSD_ACT_VF%L_BCBF=LAEROSFC
  YSD_ACT_VF%L_BCFF=LAEROSFC
  YSD_ACT_VF%L_BCGF=LAEROSFC.AND.LFIRE
  YSD_ACT_VF%L_OMBF=LAEROSFC
  YSD_ACT_VF%L_OMFF=LAEROSFC
  YSD_ACT_VF%L_OMGF=LAEROSFC.AND.LFIRE
  YSD_ACT_VF%L_SO2L=LAEROSFC
  YSD_ACT_VF%L_SO2H=LAEROSFC
  YSD_ACT_VF%L_SOGF=LAEROSFC.AND.LFIRE
  YSD_ACT_VF%L_VOLC=LAEROSFC
  YSD_ACT_VF%L_VOLE=LAEROSFC
  YSD_ACT_VF%L_SOA=LAEROSFC
  YSD_ACT_VF%L_GRGFLX=LGRGSFC.AND.(NGRG>0)
  YSD_ACT_VF%L_TRACFLX=(NTRAC>0)
  YSD_ACT_VF%L_CO2O=LCO2SFC
  YSD_ACT_VF%L_CO2B=LCO2SFC
  YSD_ACT_VF%L_CO2A=LCO2SFC
  YSD_ACT_VF%L_CO2F=LCO2SFC.AND.LFIRE
  YSD_ACT_VF%L_CH4AG=LCH4SFC
  YSD_ACT_VF%L_CH4F=LCH4SFC.AND.LFIRE
  YSD_ACT_VF%L_SF6=LSF6SFC
  YSD_ACT_VF%L_FP1=LFPART2
  YSD_ACT_VF%L_NUDM=.FALSE.
  YSD_ACT_VF%L_SSS=LESMOSS
ELSE
  YSD_ACT_VF%L_Z0F=LMPHYS.AND.(.NOT.(LMSE.OR.LLASSIMAROME))
  YSD_ACT_VF%L_ALBF=LMPHYS.AND.((.NOT.(LMSE.OR.LLASSIMAROME)).OR.(NSURFEXCTL < NSURFEXCTLMAX))
  YSD_ACT_VF%L_EMISF=LMPHYS.AND.((.NOT.(LMSE.OR.LLASSIMAROME)).OR.(NSURFEXCTL < NSURFEXCTLMAX))
  YSD_ACT_VF%L_GETRL=LMPHYS.AND.(.NOT.((LFPART2.AND.LMSE).OR.LLASSIMAROME))
  YSD_ACT_VF%L_LSM=LL_CALL_VDIFLCZ.OR.LMPHYS
  YSD_ACT_VF%L_VEG=LMPHYS.AND.(.NOT.((LMSE.AND.LFPART2).OR.LLASSIMAROME))
  YSD_ACT_VF%L_VRLAN=LMPHYS.AND.(.NOT.((LFPART2.AND.LMSE).OR.LLASSIMAROME))
  YSD_ACT_VF%L_VRLDI=LMPHYS.AND.(.NOT.((LFPART2.AND.LMSE).OR.LLASSIMAROME))
  YSD_ACT_VF%L_SIG=.FALSE.
  YSD_ACT_VF%L_ALBSF=LMPHYS.AND.(.NOT.LMSE).AND.LSOLV.AND.LSNV
  YSD_ACT_VF%L_LAN=.FALSE.
  YSD_ACT_VF%L_SST=.FALSE.
  YSD_ACT_VF%L_LZ0H=.FALSE.
  YSD_ACT_VF%L_CVL=.FALSE.
  YSD_ACT_VF%L_CVH=.FALSE.
  YSD_ACT_VF%L_TVL=.FALSE.
  YSD_ACT_VF%L_TVH=.FALSE.
  YSD_ACT_VF%L_LAIL=.FALSE.
  YSD_ACT_VF%L_LAIH=.FALSE.
  YSD_ACT_VF%L_SOTY=.FALSE.
  YSD_ACT_VF%L_CI=.FALSE.
  YSD_ACT_VF%L_UCUR=LMPHYS.AND.(.NOT.LMSE).AND.LCURR
  YSD_ACT_VF%L_VCUR=LMPHYS.AND.(.NOT.LMSE).AND.LCURR
  YSD_ACT_VF%L_Z0RLF=LMPHYS.AND.(.NOT.LMSE).AND.LSOLV.AND.LSNV
  YSD_ACT_VF%L_SDFOR=.FALSE.
  YSD_ACT_VF%L_ALUVP=.FALSE.
  YSD_ACT_VF%L_ALUVD=.FALSE.
  YSD_ACT_VF%L_ALNIP=.FALSE.
  YSD_ACT_VF%L_ALNID=.FALSE.
  YSD_ACT_VF%L_AERDEP=.FALSE.
  YSD_ACT_VF%L_AERLTS=.FALSE.
  YSD_ACT_VF%L_AERSCC=.FALSE.
  YSD_ACT_VF%L_BCBF=.FALSE.
  YSD_ACT_VF%L_BCFF=.FALSE.
  YSD_ACT_VF%L_BCGF=.FALSE.
  YSD_ACT_VF%L_OMBF=.FALSE.
  YSD_ACT_VF%L_OMFF=.FALSE.
  YSD_ACT_VF%L_OMGF=.FALSE.
  YSD_ACT_VF%L_SO2L=.FALSE.
  YSD_ACT_VF%L_SO2H=.FALSE.
  YSD_ACT_VF%L_SOGF=.FALSE.
  YSD_ACT_VF%L_VOLC=.FALSE.
  YSD_ACT_VF%L_VOLE=.FALSE.
  YSD_ACT_VF%L_SOA=.FALSE.
  YSD_ACT_VF%L_GRGFLX=.FALSE.
  YSD_ACT_VF%L_TRACFLX=.FALSE.
  YSD_ACT_VF%L_CO2O=.FALSE.
  YSD_ACT_VF%L_CO2B=.FALSE.
  YSD_ACT_VF%L_CO2A=.FALSE.
  YSD_ACT_VF%L_CO2F=.FALSE.
  YSD_ACT_VF%L_CH4AG=.FALSE.
  YSD_ACT_VF%L_CH4F=.FALSE.
  YSD_ACT_VF%L_SF6=.FALSE.
  YSD_ACT_VF%L_FP1=LFPART2
  YSD_ACT_VF%L_NUDM=LNDRIV
  YSD_ACT_VF%L_SSS=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVSF=0
IF (YSD_ACT_VF%L_Z0F) IVSF=IVSF+1
IF (YSD_ACT_VF%L_ALBF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_EMISF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_GETRL) IVSF=IVSF+1
IF (YSD_ACT_VF%L_LSM) IVSF=IVSF+1
IF (YSD_ACT_VF%L_VEG) IVSF=IVSF+1
IF (YSD_ACT_VF%L_VRLAN) IVSF=IVSF+1
IF (YSD_ACT_VF%L_VRLDI) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SIG) IVSF=IVSF+1
IF (YSD_ACT_VF%L_ALBSF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_LAN) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SST) IVSF=IVSF+1
IF (YSD_ACT_VF%L_LZ0H) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CVL) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CVH) IVSF=IVSF+1
IF (YSD_ACT_VF%L_TVL) IVSF=IVSF+1
IF (YSD_ACT_VF%L_TVH) IVSF=IVSF+1
IF (YSD_ACT_VF%L_LAIL) IVSF=IVSF+1
IF (YSD_ACT_VF%L_LAIH) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SOTY) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CI) IVSF=IVSF+1
IF (YSD_ACT_VF%L_UCUR) IVSF=IVSF+1
IF (YSD_ACT_VF%L_VCUR) IVSF=IVSF+1
IF (YSD_ACT_VF%L_Z0RLF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SDFOR) IVSF=IVSF+1
IF (YSD_ACT_VF%L_ALUVP) IVSF=IVSF+1
IF (YSD_ACT_VF%L_ALUVD) IVSF=IVSF+1
IF (YSD_ACT_VF%L_ALNIP) IVSF=IVSF+1
IF (YSD_ACT_VF%L_ALNID) IVSF=IVSF+1
IF (YSD_ACT_VF%L_AERDEP) IVSF=IVSF+1
IF (YSD_ACT_VF%L_AERLTS) IVSF=IVSF+1
IF (YSD_ACT_VF%L_AERSCC) IVSF=IVSF+1
IF (YSD_ACT_VF%L_BCBF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_BCFF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_BCGF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_OMBF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_OMFF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_OMGF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SO2L) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SO2H) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SOGF) IVSF=IVSF+1
IF (YSD_ACT_VF%L_VOLC) IVSF=IVSF+1
IF (YSD_ACT_VF%L_VOLE) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SOA) IVSF=IVSF+1
IF (YSD_ACT_VF%L_GRGFLX) IVSF=IVSF+NGRG
IF (YSD_ACT_VF%L_TRACFLX) IVSF=IVSF+NTRAC
IF (YSD_ACT_VF%L_CO2O) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CO2B) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CO2A) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CO2F) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CH4AG) IVSF=IVSF+1
IF (YSD_ACT_VF%L_CH4F) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SF6) IVSF=IVSF+1
IF (YSD_ACT_VF%L_FP1) IVSF=IVSF+1
IF (YSD_ACT_VF%L_NUDM) IVSF=IVSF+1
IF (YSD_ACT_VF%L_SSS) IVSF=IVSF+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VF%YVF)
IF(ASSOCIATED(YSD_VF%YVF)) DEALLOCATE(YSD_VF%YVF)
ALLOCATE(YSD_VF%YVF(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VFD,YSD_VF%YVF,IVSF,.FALSE.,'VARSF - SD_VF   ')
YSD_VF%YZ0F   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALBF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YEMISF => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YGETRL => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLSM   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVEG   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVRLAN => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVRLDI => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSIG   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALBSF => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLAN   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSST   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLZ0H  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCVL   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCVH   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YTVL   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YTVH   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLAIL  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YLAIH  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOTY  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCI    => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YUCUR  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVCUR  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YZ0RLF => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSDFOR => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALUVP => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALUVD => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALNIP => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YALNID => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YAERDEP=> YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YAERLTS=> YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YAERSCC=> YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YBCBF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YBCFF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YBCGF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YOMBF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YOMFF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YOMGF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSO2L  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSO2H  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOGF  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVOLC  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YVOLE  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSOA   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YGRGFLX  => YSD_VF%YVF(JPMAXSFLDS:JPMAXSFLDS)
YSD_VF%YTRACFLX  => YSD_VF%YVF(JPMAXSFLDS:JPMAXSFLDS)
YSD_VF%YCO2O  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCO2B  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCO2A  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCO2F  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCH4AG => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YCH4F  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSF6   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YFP1   => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YNUDM  => YSD_VF%YVF(JPMAXSFLDS)
YSD_VF%YSSS   => YSD_VF%YVF(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSD_ACT_VF%L_Z0F) THEN
    YSD_VF%YZ0F => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YZ0F  ,PDEFAULT=0.01_JPRB,&
     & KGRIB=NGRBSR  ,CDNAME='SURFZ0.FOIS.G   ',KTRAJ=2,KREQIN=IREQINDF)
  ENDIF
  IF (YSD_ACT_VF%L_ALBF) THEN
    YSD_VF%YALBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALBF ,PDEFAULT=0.5_JPRB,&
     & KGRIB=NGRBAL  ,CDNAME='SURFALBEDO      ',KTRAJ=2,KREQIN=IREQINDF)
  ENDIF
  IF (YSD_ACT_VF%L_EMISF) THEN
    YSD_VF%YEMISF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YEMISF,PDEFAULT=0.995_JPRB,&
     & CDNAME='SURFEMISSIVITE  ',KTRAJ=2,KREQIN=-1)
  ENDIF
  IF (YSD_ACT_VF%L_GETRL) THEN
    YSD_VF%YGETRL => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YGETRL,&
     &KGRIB=NGRBSDOR,CDNAME='SURFET.GEOPOTENT',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_LSM) THEN
    YSD_VF%YLSM => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLSM  ,&
     & KGRIB=NGRBLSM ,CDNAME='SURFIND.TERREMER',KTRAJ=2,KREQIN=1)
  ENDIF
  ! field VEG not active
  IF (YSD_ACT_VF%L_VRLAN) THEN
    YSD_VF%YVRLAN => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLAN,&
     & KGRIB=NGRBISOR,CDNAME='SURFVAR.GEOP.ANI',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_VRLDI) THEN
    YSD_VF%YVRLDI => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLDI,&
     & KGRIB=NGRBANOR,CDNAME='SURFVAR.GEOP.DIR',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SIG) THEN
    YSD_VF%YSIG => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSIG  ,&
     & KGRIB=NGRBSLOR,CDNAME='SURFMEAN SLOPE  ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_LZ0H) THEN
    YSD_VF%YLZ0H => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLZ0H ,&
     & KGRIB=NGRBLSRH,CDNAME='SURFLOGZ0H      ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CVL) THEN
    YSD_VF%YCVL => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCVL  ,&
     & KGRIB=NGRBCVL ,CDNAME='LOW VEG. COVER  ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CVH) THEN
    YSD_VF%YCVH => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCVH  ,PDEFAULT=9.21_JPRB,&
     & KGRIB=NGRBCVH ,CDNAME='HIGH VEG. COVER ',KTRAJ=2,KREQIN=IREQINDF)
  ENDIF
  IF (YSD_ACT_VF%L_TVL) THEN
    YSD_VF%YTVL => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YTVL  ,&
     & KGRIB=NGRBTVL ,CDNAME='LOW VEG. TYPE   ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_TVH) THEN
    YSD_VF%YTVH => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YTVH  ,&
     & KGRIB=NGRBTVH ,CDNAME='HIGH VEG. TYPE  ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_LAIL) THEN
    YSD_VF%YLAIL => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLAIL  ,&
     & KGRIB=NGRBLAIL ,CDNAME='LOW VEG. LAI    ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_LAIH) THEN
    YSD_VF%YLAIH => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLAIH  ,&
     & KGRIB=NGRBLAIH ,CDNAME='HIGH VEG. LAI   ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SOTY) THEN
    YSD_VF%YSOTY => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOTY ,&
     & KGRIB=NGRBSLT ,CDNAME='SOIL TYPE       ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CI) THEN
    YSD_VF%YCI => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCI   ,&
     & KGRIB=NGRBCI  ,CDNAME='SEA ICE COVER   ',KTRAJ=2,KREQIN=1)
  ENDIF
  ! field ALBSF not active
  ! field LAN not active
  IF (YSD_ACT_VF%L_SST) THEN
    YSD_VF%YSST => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSST  ,PDEFAULT=295.0_JPRB,&
     & KGRIB=NGRBSSTK ,CDNAME='SEA SURFACE TEMP',KTRAJ=2,KREQIN=IREQINDF)
  ENDIF
  IF (YSD_ACT_VF%L_UCUR) THEN
    YSD_VF%YUCUR => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YUCUR ,PDEFAULT=0.0_JPRB,&
     & KGRIB=NGRBUCUR,CDNAME='UCOM OCEAN CURRENT',KTRAJ=2,KREQIN=IREQINCUR)
  ENDIF
  IF (YSD_ACT_VF%L_VCUR) THEN
    YSD_VF%YVCUR => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVCUR ,PDEFAULT=0.0_JPRB,&
     & KGRIB=NGRBVCUR,CDNAME='VCOM OCEAN CURRENT',KTRAJ=2,KREQIN=IREQINCUR)
  ENDIF
  ! field Z0RLF not active
  IF (YSD_ACT_VF%L_SDFOR) THEN
    YSD_VF%YSDFOR => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSDFOR,&
     & KGRIB=NGRBSDFOR,CDNAME='SD FILTRED OROG ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_ALUVP) THEN
    YSD_VF%YALUVP => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALUVP,&
     & KGRIB=NGRBALUVP,CDNAME='M_ALB.UVis PARAL',KTRAJ=2,KREQIN=1)
  ENDIF  
  IF (YSD_ACT_VF%L_ALUVD) THEN
    YSD_VF%YALUVD => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALUVD,&
     & KGRIB=NGRBALUVD,CDNAME='M_ALB.UVis DIFFU',KTRAJ=2,KREQIN=1)
  ENDIF  
  IF (YSD_ACT_VF%L_ALNIP) THEN
    YSD_VF%YALNIP => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALNIP,&
     & KGRIB=NGRBALNIP,CDNAME='M_ALB.N-IR PARAL',KTRAJ=2,KREQIN=1)
  ENDIF  
  IF (YSD_ACT_VF%L_ALNID) THEN
    YSD_VF%YALNID => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALNID,&
     & KGRIB=NGRBALNID,CDNAME='M_ALB.N-IR DIFFU',KTRAJ=2,KREQIN=1)
  ENDIF  
  IF (YSD_ACT_VF%L_AERDEP) THEN
    YSD_VF%YAERDEP => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YAERDEP,&
     & KGRIB=NGRBAERDEP,CDNAME='Dust emiss poten',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_AERLTS) THEN
    YSD_VF%YAERLTS => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YAERLTS,&
     & KGRIB=NGRBAERLTS,CDNAME='Lift thresh sped',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_AERSCC) THEN
    YSD_VF%YAERSCC => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YAERSCC,&
     & KGRIB=NGRBAERSCC,CDNAME='Soil clay contnt',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_BCBF) THEN
    YSD_VF%YBCBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YBCBF,&
     & KGRIB=NGRBAERLS01,CDNAME='BlckCarbn biogen',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_BCFF) THEN
    YSD_VF%YBCFF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YBCFF,&
     & KGRIB=NGRBAERLS02,CDNAME='BlckCarbn fossil',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_BCGF) THEN
    YSD_VF%YBCGF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YBCGF,&
     & KGRIB=NGRBAERLS10,CDNAME='BlackCarbon GFED',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_OMBF) THEN
    YSD_VF%YOMBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YOMBF,&
     & KGRIB=NGRBAERLS03,CDNAME='Organic biogenic',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_OMFF) THEN
    YSD_VF%YOMFF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YOMFF,&
     & KGRIB=NGRBAERLS04,CDNAME='Organic fossil  ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_OMGF) THEN
    YSD_VF%YOMGF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YOMGF,&
     & KGRIB=NGRBAERLS11,CDNAME='Organic GFED    ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SO2L) THEN
    YSD_VF%YSO2L => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSO2L,&
     & KGRIB=NGRBAERLS05,CDNAME='Sulphate low-lvl',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SO2H) THEN
    YSD_VF%YSO2H => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSO2H,&
     & KGRIB=NGRBAERLS06,CDNAME='Sulphate highlvl',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SOGF) THEN
    YSD_VF%YSOGF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOGF,&
     & KGRIB=NGRBAERLS12,CDNAME='Sulphate GFED   ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_VOLC) THEN
    YSD_VF%YVOLC => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVOLC,&
     & KGRIB=NGRBAERLS07,CDNAME='Volcanic continu',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_VOLE) THEN
    YSD_VF%YVOLE => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVOLE,&
     & KGRIB=NGRBAERLS08,CDNAME='Volcanic explosi',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SOA) THEN
    YSD_VF%YSOA => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSOA,&
     & KGRIB=NGRBAERLS09,CDNAME='SecondOrganicMatt',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_GRGFLX) THEN 
    YSD_VF%YGRGFLX => YSD_VF%YVF(YSD_VFD%IPTR:YSD_VFD%IPTR+NGRG-1)
    DO JGRG=1,NGRG
      IREQIN=1
      IF (YGRG(JGRG)%IGRIBSFC < 0 .OR. YGRG(JGRG)%NCOUPLO4 > 0 ) IREQIN=0
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YGRGFLX(JGRG),KGRIB=YGRG(JGRG)%IGRIBSFC, &
       &    CDNAME=TRIM(YGRG(JGRG)%CNAME)//'  SURFACE FLX ', KTRAJ=2,KREQIN=IREQIN) 
    ENDDO
  ENDIF
  IF(YSD_ACT_VF%L_TRACFLX) THEN
    YSD_VF%YTRACFLX => YSD_VF%YVF(YSD_VFD%IPTR:YSD_VFD%IPTR+NTRAC-1)
    DO JTRAC=1,NTRAC
      IREQIN=1
      IF (YTRAC(JTRAC)%IGRIBSFC < 0 ) IREQIN=0
      CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YTRACFLX(JTRAC),KGRIB=YTRAC(JTRAC)%IGRIBSFC, &
       & CDNAME=TRIM(YTRAC(JTRAC)%CNAME)//'  SURFACE FLX ', KTRAJ=2,KREQIN=IREQIN) 
    ENDDO
  ENDIF
  IF (YSD_ACT_VF%L_CO2O) THEN
    YSD_VF%YCO2O => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2O ,&
     & KGRIB=NGRBCO2OF ,CDNAME='CO2 SURF - OCEAN',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CO2B) THEN
    YSD_VF%YCO2B => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2B ,&
     & KGRIB=NGRBCO2NBF ,CDNAME='CO2 SURF - BIOSF',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CO2A) THEN
    YSD_VF%YCO2A => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2A ,&
     & KGRIB=NGRBCO2APF ,CDNAME='CO2 SURF - ANTH ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CO2F) THEN
    YSD_VF%YCO2F => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCO2F ,&
     & KGRIB=NGRBCO2FIRE ,CDNAME='CO2 FIRE EMISS. ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CH4AG) THEN
    YSD_VF%YCH4AG => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCH4AG ,&
     & KGRIB=NGRBCH4F ,CDNAME='CH4 SURF. FLUXES',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_CH4F) THEN
    YSD_VF%YCH4F  => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YCH4F ,&
     & KGRIB=NGRBCH4FIRE ,CDNAME='CH4 FIRE EMISS.  ',KTRAJ=2,KREQIN=1)
  ENDIF
  IF (YSD_ACT_VF%L_SF6) THEN
    YSD_VF%YSF6 => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSF6 ,&
     & KGRIB=NGRBSF6APF ,CDNAME='SF6 SURF - ANTH ',KTRAJ=2,KREQIN=1)
  ENDIF
  ! field NUDM not active
  IF (YSD_ACT_VF%L_SSS) THEN
    YSD_VF%YSSS => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YSSS ,KGRIB=NGRBSSS ,CDNAME='SEA SFC SALINITY',KREQIN=1)
  ENDIF
ELSE
  IF (YSD_ACT_VF%L_Z0F) THEN
    YSD_VF%YZ0F => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YZ0F  ,KGRIB=NGRBSR  ,CDNAME='SURFZ0.FOIS.G   ',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_ALBF) THEN
    YSD_VF%YALBF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALBF ,KGRIB=NGRBAL  ,CDNAME='SURFALBEDO      ',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_EMISF) THEN
    YSD_VF%YEMISF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YEMISF,17      ,CDNAME='SURFEMISSIVITE  ',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_GETRL) THEN
    YSD_VF%YGETRL => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YGETRL,KGRIB=NGRBSDOR,CDNAME='SURFET.GEOPOTENT',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_LSM) THEN
    YSD_VF%YLSM => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YLSM  ,KGRIB=NGRBLSM ,CDNAME='SURFIND.TERREMER',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_VEG) THEN
    YSD_VF%YVEG => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVEG  ,CDNAME='SURFPROP.VEGETAT',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_VRLAN) THEN
    YSD_VF%YVRLAN => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLAN  ,CDNAME='SURFVAR.GEOP.ANI',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VF%L_VRLDI) THEN
    YSD_VF%YVRLDI => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVRLDI  ,CDNAME='SURFVAR.GEOP.DIR',KTRAJ=2)
  ENDIF
  ! field SIG not active
  IF (YSD_ACT_VF%L_ALBSF) THEN
    YSD_VF%YALBSF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YALBSF  ,CDNAME='SURFALBEDO.COMPL')
  ENDIF
  ! fields LAN to CI not active
  IF (YSD_ACT_VF%L_UCUR) THEN
    YSD_VF%YUCUR => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YUCUR ,CDNAME='SURFU.OCEAN.CURR', &
   & KGRIB=INOGRIB ,PDEFAULT=0._JPRB, KREQIN=-1)
  ENDIF
  IF (YSD_ACT_VF%L_VCUR) THEN
    YSD_VF%YVCUR => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YVCUR ,CDNAME='SURFV.OCEAN.CURR', &
   & KGRIB=INOGRIB ,PDEFAULT=0._JPRB, KREQIN=-1)
  ENDIF
  IF (YSD_ACT_VF%L_Z0RLF) THEN
    YSD_VF%YZ0RLF => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YZ0RLF  ,CDNAME='SURFZ0REL.FOIS.G')
  ENDIF
  ! fields SDFOR to SF6 not active
  IF (YSD_ACT_VF%L_NUDM) THEN
    YSD_VF%YNUDM => YSD_VF%YVF(YSD_VFD%IPTR)
    CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YNUDM ,KGRIB=INOGRIB ,CDNAME='SURFNUD.MASK    ')
  ENDIF
  ! field SSS not active
ENDIF
IF(YSD_ACT_VF%L_FP1) THEN
  YSD_VF%YFP1 => YSD_VF%YVF(YSD_VFD%IPTR)
  CALL SETUP_SFLP2(YSD_VFD,YSD_VF%YFP1,KGRIB=NGRBZ,CDNAME='SURFGEOPOTENTIEL',KTRAJ=0)
ENDIF


! * Group VP=VCLIP

IVCLIP=0
! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_VP%L_TPC=.FALSE.
  YSD_ACT_VP%L_WPC=.FALSE.
ELSE
  YSD_ACT_VP%L_TPC=LRELAXT
  IVCLIP=IVCLIP+1
  YSD_ACT_VP%L_WPC=LRELAXW
  IVCLIP=IVCLIP+1
ENDIF

! - compute number of fields allocated in this group:

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VP%YVP)
IF(ASSOCIATED(YSD_VP%YVP)) DEALLOCATE(YSD_VP%YVP)
ALLOCATE(YSD_VP%YVP(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VPD,YSD_VP%YVP,IVCLIP,.FALSE.,'VCLIP - SD_VP   ')
YSD_VP%YTPC => YSD_VP%YVP(JPMAXSFLDS)
YSD_VP%YWPC => YSD_VP%YVP(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active
ELSE
  IF (YSD_ACT_VP%L_TPC) THEN
    YSD_VP%YTPC => YSD_VP%YVP(YSD_VPD%IPTR)
    CALL SETUP_SFLP2(YSD_VPD,YSD_VP%YTPC,CDNAME='RELATEMPERATURE ')
  ENDIF
  IF (YSD_ACT_VP%L_WPC) THEN
    YSD_VP%YWPC => YSD_VP%YVP(YSD_VPD%IPTR)
    CALL SETUP_SFLP2(YSD_VPD,YSD_VP%YWPC,CDNAME='RELARESERV.EAU  ')
  ENDIF
ENDIF


! * Group VV=VCLIV

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_VV%L_ARG=.FALSE.
  YSD_ACT_VV%L_SAB=.FALSE.
  YSD_ACT_VV%L_D2=.FALSE.
  YSD_ACT_VV%L_IVEG=.FALSE.
  YSD_ACT_VV%L_RSMIN=.FALSE.
  YSD_ACT_VV%L_LAI=.FALSE.
  YSD_ACT_VV%L_HV=.FALSE.
  YSD_ACT_VV%L_Z0H=.FALSE.
  YSD_ACT_VV%L_ALS=.FALSE.
  YSD_ACT_VV%L_ALV=.FALSE.
ELSE
  YSD_ACT_VV%L_ARG=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.(LSOLV.OR.LMSE)
  YSD_ACT_VV%L_SAB=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.(LSOLV.OR.LMSE)
  YSD_ACT_VV%L_D2=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.(LSOLV.OR.LMSE)
  YSD_ACT_VV%L_IVEG=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV
  YSD_ACT_VV%L_RSMIN=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV
  YSD_ACT_VV%L_LAI=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV
  YSD_ACT_VV%L_HV=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV
  YSD_ACT_VV%L_Z0H=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV
  YSD_ACT_VV%L_ALS=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV.AND.LVGSN
  YSD_ACT_VV%L_ALV=LMPHYS.AND.(.NOT.LLASSIMAROME).AND.LSOLV.AND.LVGSN
ENDIF

! - compute number of fields allocated in this group:
IVCLIV=0
IF (YSD_ACT_VV%L_ARG) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_SAB) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_D2) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_IVEG) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_RSMIN) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_LAI) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_HV) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_Z0H) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_ALS) IVCLIV=IVCLIV+1
IF (YSD_ACT_VV%L_ALV) IVCLIV=IVCLIV+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VV%YVV)
IF(ASSOCIATED(YSD_VV%YVV)) DEALLOCATE(YSD_VV%YVV)
ALLOCATE(YSD_VV%YVV(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VVD,YSD_VV%YVV,IVCLIV,.FALSE.,'VCLIV - SD_VV   ')
YSD_VV%YARG   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YSAB   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YD2    => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YIVEG  => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YRSMIN => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YLAI   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YHV    => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YZ0H   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YALS   => YSD_VV%YVV(JPMAXSFLDS)
YSD_VV%YALV   => YSD_VV%YVV(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active
ELSE
  IF (YSD_ACT_VV%L_ARG) THEN
    YSD_VV%YARG   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YARG,CDNAME='SURFPROP.ARGILE ',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VV%L_SAB) THEN
    YSD_VV%YSAB   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YSAB,CDNAME='SURFPROP.SABLE  ',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VV%L_D2) THEN
    YSD_VV%YD2    => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YD2,CDNAME='SURFEPAIS.SOL   ')
  ENDIF
  IF (YSD_ACT_VV%L_IVEG) THEN
    YSD_VV%YIVEG  => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YIVEG,CDNAME='SURFIND.VEG.DOMI')
  ENDIF
  IF (YSD_ACT_VV%L_RSMIN) THEN
    YSD_VV%YRSMIN => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YRSMIN,CDNAME='SURFRESI.STO.MIN')
  ENDIF
  IF (YSD_ACT_VV%L_LAI) THEN
    YSD_VV%YLAI   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YLAI,CDNAME='SURFIND.FOLIAIRE')
  ENDIF
  IF (YSD_ACT_VV%L_HV) THEN
    YSD_VV%YHV    => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YHV,CDNAME='SURFRES.EVAPOTRA',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VV%L_Z0H) THEN
    YSD_VV%YZ0H   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YZ0H,CDNAME='SURFGZ0.THERM   ',KTRAJ=2)
  ENDIF
  IF (YSD_ACT_VV%L_ALS) THEN
    YSD_VV%YALS   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YALS,CDNAME='SURFALBEDO.SOLNU')
  ENDIF
  IF (YSD_ACT_VV%L_ALV) THEN
    YSD_VV%YALV   => YSD_VV%YVV(YSD_VVD%IPTR)
    CALL SETUP_SFLP2(YSD_VVD,YSD_VV%YALV,CDNAME='SURFALBEDO.VEG  ')
  ENDIF
ENDIF


! * Group VN=VCLIN

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_VN%L_TOP=.TRUE.
  YSD_ACT_VN%L_BAS=.TRUE.
  YSD_ACT_VN%L_ACPR=.TRUE.
  YSD_ACT_VN%L_ACCPR=.TRUE.
ELSE
  YSD_ACT_VN%L_TOP=.FALSE.
  YSD_ACT_VN%L_BAS=.FALSE.
  YSD_ACT_VN%L_ACPR=.FALSE.
  YSD_ACT_VN%L_ACCPR=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVCLIN=0
IF (YSD_ACT_VN%L_TOP) IVCLIN=IVCLIN+1
IF (YSD_ACT_VN%L_BAS) IVCLIN=IVCLIN+1
IF (YSD_ACT_VN%L_ACPR) IVCLIN=IVCLIN+1
IF (YSD_ACT_VN%L_ACCPR) IVCLIN=IVCLIN+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VN%YVN)
IF(ASSOCIATED(YSD_VN%YVN)) DEALLOCATE(YSD_VN%YVN)
ALLOCATE(YSD_VN%YVN(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VND,YSD_VN%YVN,IVCLIN,.FALSE.,'VCLIN - SD_VN   ')
YSD_VN%YTOP  => YSD_VN%YVN(JPMAXSFLDS)
YSD_VN%YBAS  => YSD_VN%YVN(JPMAXSFLDS)
YSD_VN%YACPR => YSD_VN%YVN(JPMAXSFLDS)
YSD_VN%YACCPR => YSD_VN%YVN(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSD_ACT_VN%L_TOP) THEN
    YSD_VN%YTOP => YSD_VN%YVN(YSD_VND%IPTR)
    CALL SETUP_SFLP2(YSD_VND,YSD_VN%YTOP)
  ENDIF
  IF (YSD_ACT_VN%L_BAS) THEN
    YSD_VN%YBAS => YSD_VN%YVN(YSD_VND%IPTR)
    CALL SETUP_SFLP2(YSD_VND,YSD_VN%YBAS)
  ENDIF
  IF (YSD_ACT_VN%L_ACPR) THEN
    YSD_VN%YACPR => YSD_VN%YVN(YSD_VND%IPTR)
    CALL SETUP_SFLP2(YSD_VND,YSD_VN%YACPR)
  ENDIF
  IF (YSD_ACT_VN%L_ACCPR) THEN
    YSD_VN%YACCPR => YSD_VN%YVN(YSD_VND%IPTR)
    CALL SETUP_SFLP2(YSD_VND,YSD_VN%YACCPR)
  ENDIF
ELSE
  ! no field currently active
ENDIF


! * Group VH=VCLIH

! - define which field must be active:
IVCLIH=0
IF(LECMWF) THEN
  YSD_ACT_VH%L_TCCH=.FALSE.
  YSD_ACT_VH%L_SCCH=.FALSE.
  YSD_ACT_VH%L_BCCH=.FALSE.
  YSD_ACT_VH%L_PBLH=.FALSE.
  YSD_ACT_VH%L_SPSH=.FALSE.
  YSD_ACT_VH%L_QSH =.FALSE.
ELSE
  YSD_ACT_VH%L_TCCH=LMPHYS.AND.(LNEBCO.OR.LGWDC)
  IVCLIH=IVCLIH+1
  YSD_ACT_VH%L_SCCH=LMPHYS.AND.(LNEBCO.OR.LGWDC)
  IVCLIH=IVCLIH+1
  YSD_ACT_VH%L_BCCH=LMPHYS.AND.(LNEBCO.OR.LGWDC)
  IVCLIH=IVCLIH+1
  YSD_ACT_VH%L_PBLH=LMPHYS.AND.(LNEBR.OR.(TRIM(CGMIXLEN) == 'TM'))
  IVCLIH=IVCLIH+1
  YSD_ACT_VH%L_SPSH=LMPHYS.AND.LPHSPSH
  IVCLIH=IVCLIH+1
  YSD_ACT_VH%L_QSH =LMPHYS.AND.LCOEFKTKE
  IVCLIH=IVCLIH+1
ENDIF

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VH%YVH)
IF(ASSOCIATED(YSD_VH%YVH)) DEALLOCATE(YSD_VH%YVH)
ALLOCATE(YSD_VH%YVH(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VHD,YSD_VH%YVH,IVCLIH,.FALSE.,'VCLIH - SD_VH   ')
YSD_VH%YTCCH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YSCCH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YBCCH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YPBLH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YSPSH => YSD_VH%YVH(JPMAXSFLDS)
YSD_VH%YQSH  => YSD_VH%YVH(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active
ELSE
  IF(YSD_ACT_VH%L_TCCH) THEN
    YSD_VH%YTCCH => YSD_VH%YVH(YSD_VHD%IPTR)
    CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YTCCH,CDNAME='SURFTCCH        ')
  ENDIF
  IF(YSD_ACT_VH%L_SCCH) THEN
    YSD_VH%YSCCH => YSD_VH%YVH(YSD_VHD%IPTR)
    CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YSCCH,CDNAME='SURFSCCH        ')
  ENDIF
  IF(YSD_ACT_VH%L_BCCH) THEN
    YSD_VH%YBCCH => YSD_VH%YVH(YSD_VHD%IPTR)
    CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YBCCH,CDNAME='SURFSCCH        ')
  ENDIF
  IF(YSD_ACT_VH%L_PBLH) THEN
    YSD_VH%YPBLH => YSD_VH%YVH(YSD_VHD%IPTR)
    CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YPBLH,CDNAME='SURFSBLH        ')
  ENDIF
  IF(YSD_ACT_VH%L_SPSH) THEN
    YSD_VH%YSPSH => YSD_VH%YVH(YSD_VHD%IPTR)
    CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YSPSH,CDNAME='SURFSPSH        ')
  ENDIF
  IF(YSD_ACT_VH%L_QSH ) THEN
    YSD_VH%YQSH  => YSD_VH%YVH(YSD_VHD%IPTR)
    CALL SETUP_SFLP2(YSD_VHD,YSD_VH%YQSH ,CDNAME='SURFQSH         ')
  ENDIF
ENDIF


! * Group VA=VCLIA

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_VA%L_SEA=.FALSE.
  YSD_ACT_VA%L_LAN=.FALSE.
  YSD_ACT_VA%L_SOO=.FALSE.
  YSD_ACT_VA%L_DES=.FALSE.
  YSD_ACT_VA%L_SUL=.FALSE.
  YSD_ACT_VA%L_VOL=.FALSE.
ELSE
  YSD_ACT_VA%L_SEA=LAEROSEA
  YSD_ACT_VA%L_LAN=LAEROLAN
  YSD_ACT_VA%L_SOO=LAEROSOO
  YSD_ACT_VA%L_DES=LAERODES
  YSD_ACT_VA%L_SUL=LAEROSUL
  YSD_ACT_VA%L_VOL=LAEROVOL
ENDIF

! - compute number of fields allocated in this group:
IVCLIA=0
IF (YSD_ACT_VA%L_SEA) IVCLIA=IVCLIA+1
IF (YSD_ACT_VA%L_LAN) IVCLIA=IVCLIA+1
IF (YSD_ACT_VA%L_SOO) IVCLIA=IVCLIA+1
IF (YSD_ACT_VA%L_DES) IVCLIA=IVCLIA+1
IF (YSD_ACT_VA%L_SUL) IVCLIA=IVCLIA+1
IF (YSD_ACT_VA%L_VOL) IVCLIA=IVCLIA+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VA%YVA)
IF(ASSOCIATED(YSD_VA%YVA)) DEALLOCATE(YSD_VA%YVA)
ALLOCATE(YSD_VA%YVA(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VAD,YSD_VA%YVA,IVCLIA,.FALSE.,'VCLIA - SD_VA   ')
YSD_VA%YSEA => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YLAN => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YSOO => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YDES => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YSUL => YSD_VA%YVA(JPMAXSFLDS)
YSD_VA%YVOL => YSD_VA%YVA(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active
ELSE
  IF (YSD_ACT_VA%L_SEA) THEN
    YSD_VA%YSEA => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YSEA,CDNAME='SURFAEROS.SEA   ')
  ENDIF
  IF (YSD_ACT_VA%L_LAN) THEN
    YSD_VA%YLAN => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YLAN,CDNAME='SURFAEROS.LAND  ')
  ENDIF
  IF (YSD_ACT_VA%L_SOO) THEN
    YSD_VA%YSOO => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YSOO,CDNAME='SURFAEROS.SOOT  ')
  ENDIF
  IF (YSD_ACT_VA%L_DES) THEN
    YSD_VA%YDES => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YDES,CDNAME='SURFAEROS.DESERT')
  ENDIF
  IF (YSD_ACT_VA%L_SUL) THEN
    YSD_VA%YSUL => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YSUL,CDNAME='SURFAEROS.SULFAT')
  ENDIF
  IF (YSD_ACT_VA%L_VOL) THEN
    YSD_VA%YVOL => YSD_VA%YVA(YSD_VAD%IPTR)
    CALL SETUP_SFLP2(YSD_VAD,YSD_VA%YVOL,CDNAME='SURFAEROS.VOLCAN')
  ENDIF
ENDIF


! * Group VG=VCLIG
!   Not used for the time being but some code already exists in surface_fields_mix.F90


! * Group VC=VO3ABC

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_VC%L_A=.FALSE.
  YSD_ACT_VC%L_B=.FALSE.
  YSD_ACT_VC%L_C=.FALSE.
ELSE
  YSD_ACT_VC%L_A=LMPHYS.AND.LO3ABC
  YSD_ACT_VC%L_B=LMPHYS.AND.LO3ABC
  YSD_ACT_VC%L_C=LMPHYS.AND.LO3ABC
ENDIF

! - compute number of fields allocated in this group:
IVO3ABC=0
IF (YSD_ACT_VC%L_A) IVO3ABC=IVO3ABC+1
IF (YSD_ACT_VC%L_B) IVO3ABC=IVO3ABC+1
IF (YSD_ACT_VC%L_C) IVO3ABC=IVO3ABC+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VC%YVC)
IF(ASSOCIATED(YSD_VC%YVC)) DEALLOCATE(YSD_VC%YVC)
ALLOCATE(YSD_VC%YVC(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VCD,YSD_VC%YVC,IVO3ABC,.FALSE.,'VO3ABC - SD_VC  ')
YSD_VC%YA => YSD_VC%YVC(JPMAXSFLDS)
YSD_VC%YB => YSD_VC%YVC(JPMAXSFLDS)
YSD_VC%YC => YSD_VC%YVC(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  ! no field currently active
ELSE
  IF (YSD_ACT_VC%L_A) THEN
    YSD_VC%YA => YSD_VC%YVC(YSD_VCD%IPTR)
    CALL SETUP_SFLP2(YSD_VCD,YSD_VC%YA,CDNAME='SURFA.OF.OZONE  ')
  ENDIF
  IF (YSD_ACT_VC%L_B) THEN
    YSD_VC%YB => YSD_VC%YVC(YSD_VCD%IPTR)
    CALL SETUP_SFLP2(YSD_VCD,YSD_VC%YB,CDNAME='SURFB.OF.OZONE  ')
  ENDIF
  IF (YSD_ACT_VC%L_C) THEN
    YSD_VC%YC => YSD_VC%YVC(YSD_VCD%IPTR)
    CALL SETUP_SFLP2(YSD_VCD,YSD_VC%YC,CDNAME='SURFC.OF.OZONE  ')
  ENDIF
ENDIF


! * Group V2=VDIAGO2 (KPP)

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_V2%L_OCDEP=LEOCML
  YSD_ACT_V2%L_USTRC=LEOCML
  YSD_ACT_V2%L_VSTRC=LEOCML
ELSE
  YSD_ACT_V2%L_OCDEP=.FALSE.
  YSD_ACT_V2%L_USTRC=.FALSE.
  YSD_ACT_V2%L_VSTRC=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVO2=0
IF (YSD_ACT_V2%L_OCDEP) IVO2=IVO2+1
IF (YSD_ACT_V2%L_USTRC) IVO2=IVO2+1
IF (YSD_ACT_V2%L_VSTRC) IVO2=IVO2+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_V2%YV2)
IF(ASSOCIATED(YSD_V2%YV2)) DEALLOCATE(YSD_V2%YV2)
ALLOCATE(YSD_V2%YV2(JPMAXSFLDS))
CALL INI_SFLP2(YSD_V2D,YSD_V2%YV2,IVO2,.FALSE.,'VDIAGO2 - SD_V2 ')
YSD_V2%YOCDEP => YSD_V2%YV2(JPMAXSFLDS)
YSD_V2%YUSTRC => YSD_V2%YV2(JPMAXSFLDS)
YSD_V2%YVSTRC => YSD_V2%YV2(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSD_ACT_V2%L_OCDEP) THEN
    YSD_V2%YOCDEP => YSD_V2%YV2(YSD_V2D%IPTR)
    CALL SETUP_SFLP2(YSD_V2D,YSD_V2%YOCDEP,KGRIB=NGRBOCDEP,&
                    &CDNAME='OCEAN DEP       ',KREQIN=2 )
  ENDIF
  IF (YSD_ACT_V2%L_USTRC) THEN
    YSD_V2%YUSTRC => YSD_V2%YV2(YSD_V2D%IPTR)
    CALL SETUP_SFLP2(YSD_V2D,YSD_V2%YUSTRC,KGRIB=NGRBUSTRC,&
                    &CDNAME='CLIM TAUX       ',KREQIN=2 )
  ENDIF
  IF (YSD_ACT_V2%L_VSTRC) THEN
    YSD_V2%YVSTRC => YSD_V2%YV2(YSD_V2D%IPTR)
    CALL SETUP_SFLP2(YSD_V2D,YSD_V2%YVSTRC,KGRIB=NGRBVSTRC,&
                    &CDNAME='CLIM TAUY       ',KREQIN=2 )
  ENDIF
ELSE
  ! no field currently active
ENDIF


! * Group V3=VDIAGO3 (KPP)

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_V3%L_DIFM  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_DIFT  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_DIFS  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_ADVT  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_ADVS  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_TRI0  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_TRI1  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_SWDK  =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_ZO    =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_HO    =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_DO    =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_HO_INV=(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_UOC   =(.NOT.LLNOSURF).AND.LEOCML
  YSD_ACT_V3%L_VOC   =(.NOT.LLNOSURF).AND.LEOCML
ELSE
  YSD_ACT_V3%L_DIFM  =.FALSE.
  YSD_ACT_V3%L_DIFT  =.FALSE.
  YSD_ACT_V3%L_DIFS  =.FALSE.
  YSD_ACT_V3%L_ADVT  =.FALSE.
  YSD_ACT_V3%L_ADVS  =.FALSE.
  YSD_ACT_V3%L_TRI0  =.FALSE.
  YSD_ACT_V3%L_TRI1  =.FALSE.
  YSD_ACT_V3%L_SWDK  =.FALSE.
  YSD_ACT_V3%L_ZO    =.FALSE.
  YSD_ACT_V3%L_HO    =.FALSE.
  YSD_ACT_V3%L_DO    =.FALSE.
  YSD_ACT_V3%L_HO_INV=.FALSE.
  YSD_ACT_V3%L_UOC   =.FALSE.
  YSD_ACT_V3%L_VOC   =.FALSE.
ENDIF


! - compute number of fields allocated in this group:
IVO3=0
IF (YSD_ACT_V3%L_DIFM  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_DIFT  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_DIFS  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_ADVT  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_ADVS  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_TRI0  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_TRI1  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_SWDK  ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_ZO    ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_HO    ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_DO    ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_HO_INV) IVO3=IVO3+1
IF (YSD_ACT_V3%L_UOC   ) IVO3=IVO3+1
IF (YSD_ACT_V3%L_VOC   ) IVO3=IVO3+1
IF (IVO3>0) THEN
  ICOM=NCOM
ELSE
  ICOM=0
ENDIF

! - call to INI_SFLP3, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_V3%YV3)
IF(ASSOCIATED(YSD_V3%YV3)) DEALLOCATE(YSD_V3%YV3)
ALLOCATE(YSD_V3%YV3(JPMAXSFLDS))
CALL INI_SFLP3(YSD_V3D,YSD_V3%YV3,IVO3,ICOM,.FALSE.,'VDIAGO3 - SD_V3 ')
YSD_V3%YDIFM   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YDIFT   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YDIFS   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YADVT   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YADVS   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YTRI0   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YTRI1   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YSWDK   => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YZO     => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YHO     => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YDO     => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YHO_INV => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YUOC    => YSD_V3%YV3(JPMAXSFLDS)
YSD_V3%YVOC    => YSD_V3%YV3(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF(YSD_ACT_V3%L_DIFM) THEN
    IGRBCODES(1:ICOM) = NGRBOCVVS
    CLNAMES  (1:ICOM) = 'OCEAN VISC     '
    YSD_V3%YDIFM => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDIFM,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_DIFT) THEN
    IGRBCODES(1:ICOM) = NGRBOCVDF
    CLNAMES  (1:ICOM) = 'OCEAN DIFF     '
    YSD_V3%YDIFT => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDIFT,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_DIFS) THEN
    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YDIFS => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDIFT,KGRIB=IGRBCODES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_ADVT) THEN
    IGRBCODES(1:ICOM) = NGRBADVT
    CLNAMES  (1:ICOM) = 'OCEAN ADVT     '
    YSD_V3%YADVT => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YADVT,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_ADVS) THEN
    IGRBCODES(1:ICOM) = NGRBADVS
    CLNAMES  (1:ICOM) = 'OCEAN ADVS     '
    YSD_V3%YADVS => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YADVS,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_TRI0) THEN
    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YTRI0 => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YTRI0,KGRIB=IGRBCODES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_TRI1) THEN
    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YTRI1 => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YTRI1,KGRIB=IGRBCODES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_SWDK) THEN
    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YSWDK => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YSWDK,KGRIB=IGRBCODES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_ZO) THEN
    IGRBCODES(1:ICOM) = NGRBOCLZ
    CLNAMES  (1:ICOM) = 'OCEAN LAYER DEP'  
    YSD_V3%YZO => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YZO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_HO) THEN
    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YHO => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YHO,KGRIB=IGRBCODES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_DO) THEN
    IGRBCODES(1:ICOM) = NGRBOCLDP
    CLNAMES  (1:ICOM) = 'OCEAN THICKNESS'  
    YSD_V3%YDO => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YDO,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_HO_INV) THEN
    IGRBCODES(1:ICOM) = INOGRIB
    YSD_V3%YHO_INV => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YHO_INV,KGRIB=IGRBCODES(1:ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_UOC) THEN
    IGRBCODES(1:ICOM) = NGRBOCUC
    CLNAMES  (1:ICOM) = 'OCN U-VEL CLIM '
    YSD_V3%YUOC => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YUOC,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))
  ENDIF
  IF(YSD_ACT_V3%L_VOC) THEN
    IGRBCODES(1:ICOM) = NGRBOCVC
    CLNAMES  (1:ICOM) = 'OCN V-VEL CLIM '
    YSD_V3%YVOC => YSD_V3%YV3(YSD_V3D%IPTR)
    CALL SETUP_SFLP3(YSD_V3D,YSD_V3%YVOC,KGRIB=IGRBCODES(1:ICOM),&
     & CDNAME=CLNAMES(1:ICOM),KREQIN=SPREAD(2,1,ICOM))
  ENDIF
ELSE
  ! no field currently active
ENDIF


! * Group VD=VDIAG

! - define which field must be active:
IF(LECMWF) THEN
  YSD_ACT_VD%L_LSP  =.TRUE.
  YSD_ACT_VD%L_CP   =.TRUE.
  YSD_ACT_VD%L_SF   =.TRUE.
  YSD_ACT_VD%L_BLD  =.TRUE.
  YSD_ACT_VD%L_SSHF =.TRUE.
  YSD_ACT_VD%L_SLHF =.TRUE.
  YSD_ACT_VD%L_NEE  =.TRUE.
  YSD_ACT_VD%L_GPP  =.TRUE.
  YSD_ACT_VD%L_REC  =.TRUE.
  YSD_ACT_VD%L_MSL  =.TRUE.
  YSD_ACT_VD%L_SP   =.TRUE.
  YSD_ACT_VD%L_TCC  =.TRUE.
  YSD_ACT_VD%L_10U  =.TRUE.
  YSD_ACT_VD%L_10V  =.TRUE.
  YSD_ACT_VD%L_2T   =.TRUE.
  YSD_ACT_VD%L_2D   =.TRUE.
  YSD_ACT_VD%L_SSR  =.TRUE.
  YSD_ACT_VD%L_STR  =.TRUE.
  YSD_ACT_VD%L_TSR  =.TRUE.
  YSD_ACT_VD%L_TTR  =.TRUE.
  YSD_ACT_VD%L_EWSS =.TRUE.
  YSD_ACT_VD%L_NSSS =.TRUE.
  YSD_ACT_VD%L_E    =.TRUE.
  YSD_ACT_VD%L_CCC  =.TRUE.
  YSD_ACT_VD%L_LCC  =.TRUE.
  YSD_ACT_VD%L_MCC  =.TRUE.
  YSD_ACT_VD%L_HCC  =.TRUE.
  YSD_ACT_VD%L_LGWS =.TRUE.
  YSD_ACT_VD%L_MGWS =.TRUE.
  YSD_ACT_VD%L_GWD  =.TRUE.
  YSD_ACT_VD%L_MX2T =.TRUE.
  YSD_ACT_VD%L_MN2T =.TRUE.
  YSD_ACT_VD%L_RO   =.TRUE.
  YSD_ACT_VD%L_SRO  =.TRUE.
  YSD_ACT_VD%L_SSRO =.TRUE.
  YSD_ACT_VD%L_ALB  =.TRUE.
  YSD_ACT_VD%L_IEWSS=.TRUE.
  YSD_ACT_VD%L_INSSS=.TRUE.
  YSD_ACT_VD%L_ISSHF=.TRUE.
  YSD_ACT_VD%L_IE   =.TRUE.
  YSD_ACT_VD%L_INEE =.TRUE.
  YSD_ACT_VD%L_IGPP =.TRUE.
  YSD_ACT_VD%L_IREC =.TRUE.
  YSD_ACT_VD%L_CSF  =.TRUE.
  YSD_ACT_VD%L_LSSF =.TRUE.
  YSD_ACT_VD%L_Z0F  =.TRUE.
  YSD_ACT_VD%L_LZ0H =.TRUE.
  YSD_ACT_VD%L_TCW  =.TRUE.
  YSD_ACT_VD%L_TCWV =.TRUE.
  YSD_ACT_VD%L_TCLW =.TRUE.
  YSD_ACT_VD%L_TCIW =.TRUE.
  YSD_ACT_VD%L_TCRW =.TRUE.
  YSD_ACT_VD%L_TCSW =.TRUE.
  YSD_ACT_VD%L_SSRD =.TRUE.
  YSD_ACT_VD%L_STRD =.TRUE.
  YSD_ACT_VD%L_BLH  =.TRUE.
  YSD_ACT_VD%L_SUND =.TRUE.
  YSD_ACT_VD%L_SPAR =.TRUE.
  YSD_ACT_VD%L_SUVB =.TRUE.
  YSD_ACT_VD%L_CAPE =.TRUE.
  YSD_ACT_VD%L_TSRC =.TRUE.
  YSD_ACT_VD%L_TTRC =.TRUE.
  YSD_ACT_VD%L_SSRC =.TRUE.
  YSD_ACT_VD%L_STRC =.TRUE.
  YSD_ACT_VD%L_ES   =.TRUE.
  YSD_ACT_VD%L_SMLT =.TRUE.
  YSD_ACT_VD%L_10FG =.TRUE.
  YSD_ACT_VD%L_LSPF =.TRUE.
  YSD_ACT_VD%L_TCO3 =.TRUE.
  YSD_ACT_VD%L_VIMD =.TRUE.
  YSD_ACT_VD%L_SPARC=.TRUE.
  YSD_ACT_VD%L_STINC=.TRUE.
  YSD_ACT_VD%L_SFDIR=.TRUE.
  YSD_ACT_VD%L_SCDIR=.TRUE.
  YSD_ACT_VD%L_CBASE=.TRUE.
  YSD_ACT_VD%L_0DEGL=.TRUE.
  YSD_ACT_VD%L_VISIH=.TRUE.
  YSD_ACT_VD%L_100U =.TRUE.
  YSD_ACT_VD%L_100V =.TRUE.
  YSD_ACT_VD%L_ZUST =.TRUE.
  YSD_ACT_VD%L_10NU =.TRUE.
  YSD_ACT_VD%L_10NV =.TRUE.
  YSD_ACT_VD%L_DNDZN=.TRUE.
  YSD_ACT_VD%L_DNDZA=.TRUE.
  YSD_ACT_VD%L_DCTB =.TRUE.
  YSD_ACT_VD%L_TPLB =.TRUE.
  YSD_ACT_VD%L_TPLT =.TRUE.
  YSD_ACT_VD%L_TCGHG=NGHG>0
  YSD_ACT_VD%L_TCGRG=NGRG>0
  YSD_ACT_VD%L_TCTRAC=NTRAC>0
  YSD_ACT_VD%L_MX2T6=N6BINS>0
  YSD_ACT_VD%L_MN2T6=N6BINS>0
  YSD_ACT_VD%L_10FG6=N6BINS>0
ELSE
  YSD_ACT_VD%L_LSP  =.FALSE.
  YSD_ACT_VD%L_CP   =.FALSE.
  YSD_ACT_VD%L_SF   =.FALSE.
  YSD_ACT_VD%L_BLD  =.FALSE.
  YSD_ACT_VD%L_SSHF =.FALSE.
  YSD_ACT_VD%L_SLHF =.FALSE.
  YSD_ACT_VD%L_NEE  =.FALSE.
  YSD_ACT_VD%L_GPP  =.FALSE.
  YSD_ACT_VD%L_REC  =.FALSE.
  YSD_ACT_VD%L_MSL  =.FALSE.
  YSD_ACT_VD%L_SP   =.FALSE.
  YSD_ACT_VD%L_TCC  =.FALSE.
  YSD_ACT_VD%L_10U  =.FALSE.
  YSD_ACT_VD%L_10V  =.FALSE.
  YSD_ACT_VD%L_2T   =.FALSE.
  YSD_ACT_VD%L_2D   =.FALSE.
  YSD_ACT_VD%L_SSR  =.FALSE.
  YSD_ACT_VD%L_STR  =.FALSE.
  YSD_ACT_VD%L_TSR  =.FALSE.
  YSD_ACT_VD%L_TTR  =.FALSE.
  YSD_ACT_VD%L_EWSS =.FALSE.
  YSD_ACT_VD%L_NSSS =.FALSE.
  YSD_ACT_VD%L_E    =.FALSE.
  YSD_ACT_VD%L_CCC  =.FALSE.
  YSD_ACT_VD%L_LCC  =.FALSE.
  YSD_ACT_VD%L_MCC  =.FALSE.
  YSD_ACT_VD%L_HCC  =.FALSE.
  YSD_ACT_VD%L_LGWS =.FALSE.
  YSD_ACT_VD%L_MGWS =.FALSE.
  YSD_ACT_VD%L_GWD  =.FALSE.
  YSD_ACT_VD%L_MX2T =.FALSE.
  YSD_ACT_VD%L_MN2T =.FALSE.
  YSD_ACT_VD%L_RO   =.FALSE.
  YSD_ACT_VD%L_SRO  =.FALSE.
  YSD_ACT_VD%L_SSRO =.FALSE.
  YSD_ACT_VD%L_ALB  =.FALSE.
  YSD_ACT_VD%L_IEWSS=.FALSE.
  YSD_ACT_VD%L_INSSS=.FALSE.
  YSD_ACT_VD%L_ISSHF=.FALSE.
  YSD_ACT_VD%L_IE   =.FALSE.
  YSD_ACT_VD%L_INEE =.FALSE.
  YSD_ACT_VD%L_IGPP =.FALSE.
  YSD_ACT_VD%L_IREC =.FALSE.
  YSD_ACT_VD%L_CSF  =.FALSE.
  YSD_ACT_VD%L_LSSF =.FALSE.
  YSD_ACT_VD%L_Z0F  =.FALSE.
  YSD_ACT_VD%L_LZ0H =.FALSE.
  YSD_ACT_VD%L_TCW  =.FALSE.
  YSD_ACT_VD%L_TCWV =.FALSE.
  YSD_ACT_VD%L_TCLW =.FALSE.
  YSD_ACT_VD%L_TCIW =.FALSE.
  YSD_ACT_VD%L_TCRW =.FALSE.
  YSD_ACT_VD%L_TCSW =.FALSE.
  YSD_ACT_VD%L_SSRD =.FALSE.
  YSD_ACT_VD%L_STRD =.FALSE.
  YSD_ACT_VD%L_BLH  =.FALSE.
  YSD_ACT_VD%L_SUND =LMPHYS.AND.(.NOT.LAROME)
  YSD_ACT_VD%L_SPAR =.FALSE.
  YSD_ACT_VD%L_SUVB =.FALSE.
  YSD_ACT_VD%L_CAPE =.FALSE.
  YSD_ACT_VD%L_TSRC =.FALSE.
  YSD_ACT_VD%L_TTRC =.FALSE.
  YSD_ACT_VD%L_SSRC =.FALSE.
  YSD_ACT_VD%L_STRC =.FALSE.
  YSD_ACT_VD%L_ES   =.FALSE.
  YSD_ACT_VD%L_SMLT =.FALSE.
  YSD_ACT_VD%L_10FG =.FALSE.
  YSD_ACT_VD%L_LSPF =.FALSE.
  YSD_ACT_VD%L_TCO3 =.FALSE.
  YSD_ACT_VD%L_VIMD =.FALSE.
  YSD_ACT_VD%L_SPARC=.FALSE.
  YSD_ACT_VD%L_STINC=.FALSE.
  YSD_ACT_VD%L_SFDIR=.FALSE.
  YSD_ACT_VD%L_SCDIR=.FALSE.
  YSD_ACT_VD%L_CBASE=.FALSE.
  YSD_ACT_VD%L_0DEGL=.FALSE.
  YSD_ACT_VD%L_VISIH=.FALSE.
  YSD_ACT_VD%L_100U =.FALSE.
  YSD_ACT_VD%L_100V =.FALSE.
  YSD_ACT_VD%L_ZUST =.FALSE.
  YSD_ACT_VD%L_10NU =.FALSE.
  YSD_ACT_VD%L_10NV =.FALSE.
  YSD_ACT_VD%L_DNDZN=.FALSE.
  YSD_ACT_VD%L_DNDZA=.FALSE.
  YSD_ACT_VD%L_DCTB =.FALSE.
  YSD_ACT_VD%L_TPLB =.FALSE.
  YSD_ACT_VD%L_TPLT =.FALSE.
  YSD_ACT_VD%L_TCGHG=.FALSE.
  YSD_ACT_VD%L_TCGRG=.FALSE.
  YSD_ACT_VD%L_TCTRAC=.FALSE.
  YSD_ACT_VD%L_MX2T6=.FALSE.
  YSD_ACT_VD%L_MN2T6=.FALSE.
  YSD_ACT_VD%L_10FG6=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVDIAG=0
IF (YSD_ACT_VD%L_LSP  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_CP   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SF   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_BLD  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SSHF ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SLHF ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_NEE  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_GPP  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_REC  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_MSL  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SP   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCC  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_10U  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_10V  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_2T   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_2D   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SSR  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_STR  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TSR  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TTR  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_EWSS ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_NSSS ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_E    ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_CCC  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_LCC  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_MCC  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_HCC  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_LGWS ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_MGWS ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_GWD  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_MX2T ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_MN2T ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_RO   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SRO  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SSRO ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_ALB  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_IEWSS) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_INSSS) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_ISSHF) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_IE   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_INEE ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_IGPP ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_IREC ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_CSF  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_LSSF ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_Z0F  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_LZ0H ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCW  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCWV ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCLW ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCIW ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCRW ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCSW ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SSRD ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_STRD ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_BLH  ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SUND ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SPAR ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SUVB ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_CAPE ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TSRC ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TTRC ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SSRC ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_STRC ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_ES   ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SMLT ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_10FG ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_LSPF ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCO3 ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_VIMD ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SPARC) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_STINC) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SFDIR) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_SCDIR) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_CBASE) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_0DEGL) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_VISIH) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_100U ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_100V ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_ZUST ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_10NU ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_10NV ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_DNDZN) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_DNDZA) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_DCTB ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TPLB ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TPLT ) IVDIAG=IVDIAG+1
IF (YSD_ACT_VD%L_TCGHG) IVDIAG=IVDIAG+NGHG
IF (YSD_ACT_VD%L_TCGRG) IVDIAG=IVDIAG+NGRG
IF (YSD_ACT_VD%L_TCTRAC) IVDIAG=IVDIAG+NTRAC
IF (YSD_ACT_VD%L_MX2T6) IVDIAG=IVDIAG+N6BINS
IF (YSD_ACT_VD%L_MN2T6) IVDIAG=IVDIAG+N6BINS
IF (YSD_ACT_VD%L_10FG6) IVDIAG=IVDIAG+N6BINS

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VD%YVD)
IF(ASSOCIATED(YSD_VD%YVD)) DEALLOCATE(YSD_VD%YVD)
ALLOCATE(YSD_VD%YVD(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VDD,YSD_VD%YVD,IVDIAG,.FALSE.,'VDIAG - SD_VD   ')
YSD_VD%YLSP   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCP    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSF    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YBLD   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSHF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSLHF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YNEE   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YGPP   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YREC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMSL   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSP    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10U   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10V   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y2T    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y2D    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTSR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTTR   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YEWSS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YNSSS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YE     => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YHCC   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLGWS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMGWS  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YGWD   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMX2T  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YMN2T  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YRO    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSRO   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSRO  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YALB   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YIEWSS => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YINSSS => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YISSHF => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YIE    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YINEE  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YIGPP  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YIREC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCSF   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLSSF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YZ0F   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLZ0H  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCW   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCWV  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCLW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCIW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCRW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCSW  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSRD  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTRD  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YBLH   => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSUND  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSPAR  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSUVB  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCAPE  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTSRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTTRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSSRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTRC  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YES    => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSMLT  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10FG  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YLSPF  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTCO3  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YVIMD  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSPARC => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSTINC => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSFDIR => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YSCDIR => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YCBASE => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y0DEGL => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YVISIH => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y100U  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y100V  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YZUST  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10NU  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%Y10NV  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YDNDZN => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YDNDZA => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YDCTB  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTPLB  => YSD_VD%YVD(JPMAXSFLDS)
YSD_VD%YTPLT  => YSD_VD%YVD(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSD_ACT_VD%L_LSP) THEN
    YSD_VD%YLSP => YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLSP,KGRIB=NGRBLSP)
  ENDIF
  IF (YSD_ACT_VD%L_CP) THEN
    YSD_VD%YCP=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCP,KGRIB=NGRBCP)
  ENDIF
  IF (YSD_ACT_VD%L_SF) THEN
    YSD_VD%YSF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSF,KGRIB=NGRBSF)
  ENDIF
  IF (YSD_ACT_VD%L_BLD) THEN
    YSD_VD%YBLD=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YBLD,KGRIB=NGRBBLD)
  ENDIF
  IF (YSD_ACT_VD%L_SSHF) THEN
    YSD_VD%YSSHF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSHF,KGRIB=NGRBSSHF)
  ENDIF
  IF (YSD_ACT_VD%L_SLHF) THEN
    YSD_VD%YSLHF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSLHF,KGRIB=NGRBSLHF)
  ENDIF
  IF (YSD_ACT_VD%L_NEE) THEN
    YSD_VD%YNEE=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YNEE,KGRIB=NGRBNEE)
  ENDIF
  IF (YSD_ACT_VD%L_GPP) THEN
    YSD_VD%YGPP=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YGPP,KGRIB=NGRBGPP)
  ENDIF
  IF (YSD_ACT_VD%L_REC) THEN
    YSD_VD%YREC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YREC,KGRIB=NGRBREC)
  ENDIF
  IF (YSD_ACT_VD%L_MSL) THEN
    YSD_VD%YMSL=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMSL,KGRIB=NGRBMSL)
  ENDIF
  IF (YSD_ACT_VD%L_SP) THEN
    YSD_VD%YSP => YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSP,KGRIB=NGRBSP)
  ENDIF
  IF (YSD_ACT_VD%L_TCC) THEN
    YSD_VD%YTCC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCC,KGRIB=NGRBTCC)
  ENDIF
  IF (YSD_ACT_VD%L_10U) THEN
    YSD_VD%Y10U=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10U,KGRIB=NGRB10U)
  ENDIF
  IF (YSD_ACT_VD%L_10V) THEN
    YSD_VD%Y10V=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10V,KGRIB=NGRB10V)
  ENDIF
  IF (YSD_ACT_VD%L_2T) THEN
    YSD_VD%Y2T=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y2T,KGRIB=NGRB2T)
  ENDIF
  IF (YSD_ACT_VD%L_2D) THEN
    YSD_VD%Y2D=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y2D,KGRIB=NGRB2D)
  ENDIF
  IF (YSD_ACT_VD%L_SSR) THEN
    YSD_VD%YSSR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSR,KGRIB=NGRBSSR)
  ENDIF
  IF (YSD_ACT_VD%L_STR) THEN
    YSD_VD%YSTR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTR,KGRIB=NGRBSTR)
  ENDIF
  IF (YSD_ACT_VD%L_TSR) THEN
    YSD_VD%YTSR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTSR,KGRIB=NGRBTSR)
  ENDIF
  IF (YSD_ACT_VD%L_TTR) THEN
    YSD_VD%YTTR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTTR,KGRIB=NGRBTTR)
  ENDIF
  IF (YSD_ACT_VD%L_EWSS) THEN
    YSD_VD%YEWSS=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YEWSS,KGRIB=NGRBEWSS)
  ENDIF
  IF (YSD_ACT_VD%L_NSSS) THEN
    YSD_VD%YNSSS=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YNSSS,KGRIB=NGRBNSSS)
  ENDIF
  IF (YSD_ACT_VD%L_E) THEN
    YSD_VD%YE=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YE,KGRIB=NGRBE)
  ENDIF
  IF (YSD_ACT_VD%L_CCC) THEN
    YSD_VD%YCCC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCCC,KGRIB=NGRBCCC)
  ENDIF
  IF (YSD_ACT_VD%L_LCC) THEN
    YSD_VD%YLCC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLCC,KGRIB=NGRBLCC)
  ENDIF
  IF (YSD_ACT_VD%L_MCC) THEN
    YSD_VD%YMCC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMCC,KGRIB=NGRBMCC)
  ENDIF
  IF (YSD_ACT_VD%L_HCC) THEN
    YSD_VD%YHCC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YHCC,KGRIB=NGRBHCC)
  ENDIF
  IF (YSD_ACT_VD%L_LGWS) THEN
    YSD_VD%YLGWS=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLGWS,KGRIB=NGRBLGWS)
  ENDIF
  IF (YSD_ACT_VD%L_MGWS) THEN
    YSD_VD%YMGWS=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMGWS,KGRIB=NGRBMGWS)
  ENDIF
  IF (YSD_ACT_VD%L_GWD) THEN
    YSD_VD%YGWD=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YGWD,KGRIB=NGRBGWD)
  ENDIF
  IF (YSD_ACT_VD%L_MX2T) THEN
    YSD_VD%YMX2T=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMX2T,KGRIB=NGRBMX2T)
  ENDIF
  IF (YSD_ACT_VD%L_MN2T) THEN
    YSD_VD%YMN2T=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMN2T,KGRIB=NGRBMN2T)
  ENDIF
  IF (YSD_ACT_VD%L_RO) THEN
    YSD_VD%YRO=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YRO,KGRIB=NGRBRO)
  ENDIF
  IF (YSD_ACT_VD%L_SRO) THEN
    YSD_VD%YSRO=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSRO,KGRIB=NGRBSRO)
  ENDIF
  IF (YSD_ACT_VD%L_SSRO) THEN
    YSD_VD%YSSRO=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSRO,KGRIB=NGRBSSRO)
  ENDIF
  IF (YSD_ACT_VD%L_ALB) THEN
    YSD_VD%YALB=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YALB,KGRIB=NGRBFAL)
  ENDIF
  IF (YSD_ACT_VD%L_IEWSS) THEN
    YSD_VD%YIEWSS=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YIEWSS,KGRIB=NGRBIEWS)
  ENDIF
  IF (YSD_ACT_VD%L_INSSS) THEN
    YSD_VD%YINSSS=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YINSSS,KGRIB=NGRBINSS)
  ENDIF
  IF (YSD_ACT_VD%L_ISSHF) THEN
    YSD_VD%YISSHF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YISSHF,KGRIB=NGRBISHF)
  ENDIF
  IF (YSD_ACT_VD%L_IE) THEN
    YSD_VD%YIE=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YIE  ,KGRIB=NGRBIE  )
  ENDIF
  IF (YSD_ACT_VD%L_INEE) THEN
    YSD_VD%YINEE=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YINEE,KGRIB=NGRBINEE)
  ENDIF
  IF (YSD_ACT_VD%L_IGPP) THEN
    YSD_VD%YIGPP=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YIGPP,KGRIB=NGRBIGPP)
  ENDIF
  IF (YSD_ACT_VD%L_IREC) THEN
    YSD_VD%YIREC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YIREC,KGRIB=NGRBIREC)
  ENDIF
  IF (YSD_ACT_VD%L_CSF) THEN
    YSD_VD%YCSF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCSF ,KGRIB=NGRBCSF )
  ENDIF
  IF (YSD_ACT_VD%L_LSSF) THEN
    YSD_VD%YLSSF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLSSF,KGRIB=NGRBLSF )
  ENDIF
  IREQIN=0
  IF(LIFSTRAJ)   IREQIN=1
  IF(NCONF==401.OR.NCONF==501) IREQIN=-1
  IF (YSD_ACT_VD%L_Z0F) THEN
    YSD_VD%YZ0F=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YZ0F  ,KGRIB=NGRBFSR, CDNAME='FC SURFACE Z0   ',KTRAJ=1,KREQIN=IREQIN)
  ENDIF
  IF (YSD_ACT_VD%L_LZ0H) THEN
    YSD_VD%YLZ0H=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLZ0H,KGRIB=NGRBFLSR, CDNAME='LOG FC SURF Z0H ',KTRAJ=1,KREQIN=IREQIN)
  ENDIF
  IF (YSD_ACT_VD%L_TCW) THEN
    YSD_VD%YTCW=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCW ,KGRIB=NGRBTCW )
  ENDIF
  IF (YSD_ACT_VD%L_TCWV) THEN
    YSD_VD%YTCWV=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCWV,KGRIB=NGRBTCWV)
  ENDIF
  IF (YSD_ACT_VD%L_TCLW) THEN
    YSD_VD%YTCLW=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCLW,KGRIB=NGRBTCLW)
  ENDIF
  IF (YSD_ACT_VD%L_TCIW) THEN
    YSD_VD%YTCIW=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCIW,KGRIB=NGRBTCIW)
  ENDIF
  IF (YSD_ACT_VD%L_TCRW) THEN
    YSD_VD%YTCRW=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCRW,KGRIB=NGRBTCRW)
  ENDIF
  IF (YSD_ACT_VD%L_TCSW) THEN
    YSD_VD%YTCSW=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCSW,KGRIB=NGRBTCSW)
  ENDIF
  IF (YSD_ACT_VD%L_SSRD) THEN
    YSD_VD%YSSRD=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSRD,KGRIB=NGRBSSRD)
  ENDIF
  IF (YSD_ACT_VD%L_STRD) THEN
    YSD_VD%YSTRD=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTRD,KGRIB=NGRBSTRD)
  ENDIF
  IF (YSD_ACT_VD%L_BLH) THEN
    YSD_VD%YBLH=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YBLH ,KGRIB=NGRBBLH )
  ENDIF
  IF (YSD_ACT_VD%L_SUND) THEN
    YSD_VD%YSUND=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSUND,KGRIB=NGRBSUND)
  ENDIF
  IF (YSD_ACT_VD%L_SPAR) THEN
    YSD_VD%YSPAR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSPAR,KGRIB=NGRBPAR)
  ENDIF
  IF (YSD_ACT_VD%L_SUVB) THEN
    YSD_VD%YSUVB=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSUVB,KGRIB=NGRBUVB)
  ENDIF
  IF (YSD_ACT_VD%L_CAPE) THEN
    YSD_VD%YCAPE=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCAPE,KGRIB=NGRBCAPE)
  ENDIF
  IF (YSD_ACT_VD%L_TSRC) THEN
    YSD_VD%YTSRC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTSRC,KGRIB=NGRBTSRC)
  ENDIF
  IF (YSD_ACT_VD%L_TTRC) THEN
    YSD_VD%YTTRC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTTRC,KGRIB=NGRBTTRC)
  ENDIF
  IF (YSD_ACT_VD%L_SSRC) THEN
    YSD_VD%YSSRC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSSRC,KGRIB=NGRBSSRC)
  ENDIF
  IF (YSD_ACT_VD%L_STRC) THEN
    YSD_VD%YSTRC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTRC,KGRIB=NGRBSTRC)
  ENDIF
  IF (YSD_ACT_VD%L_ES) THEN
    YSD_VD%YES=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YES  ,KGRIB=NGRBES  )
  ENDIF
  IF (YSD_ACT_VD%L_SMLT) THEN
    YSD_VD%YSMLT=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSMLT,KGRIB=NGRBSMLT)
  ENDIF
  IF (YSD_ACT_VD%L_10FG) THEN
    YSD_VD%Y10FG=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10FG,KGRIB=NGRB10FG)
  ENDIF
  IF (YSD_ACT_VD%L_LSPF) THEN
    YSD_VD%YLSPF=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YLSPF,KGRIB=NGRBLSPF)
  ENDIF
  IF (YSD_ACT_VD%L_TCO3) THEN
    YSD_VD%YTCO3=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCO3,KGRIB=NGRBTCO3)
  ENDIF
  IF (YSD_ACT_VD%L_VIMD) THEN
    YSD_VD%YVIMD=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YVIMD,KGRIB=NGRBVIMD)
  ENDIF
  IF (YSD_ACT_VD%L_SPARC) THEN
    YSD_VD%YSPARC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSPARC,KGRIB=NGRBPARCS)
  ENDIF
  IF (YSD_ACT_VD%L_STINC) THEN
    YSD_VD%YSTINC=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSTINC,KGRIB=NGRBTISR)
  ENDIF
  IF (YSD_ACT_VD%L_SFDIR) THEN
    YSD_VD%YSFDIR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSFDIR,KGRIB=NGRBFDIR)
  ENDIF
  IF (YSD_ACT_VD%L_SCDIR) THEN
    YSD_VD%YSCDIR=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSCDIR,KGRIB=NGRBCDIR)
  ENDIF
  IF (YSD_ACT_VD%L_CBASE) THEN
    YSD_VD%YCBASE=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YCBASE,KGRIB=NGRBCBASE)
  ENDIF
  IF (YSD_ACT_VD%L_0DEGL) THEN
    YSD_VD%Y0DEGL=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y0DEGL,KGRIB=NGRB0DEGL)
  ENDIF
  IF (YSD_ACT_VD%L_VISIH) THEN
    YSD_VD%YVISIH=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YVISIH,KGRIB=NGRBVISIH)
  ENDIF
  IF (YSD_ACT_VD%L_100U) THEN
    YSD_VD%Y100U=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y100U,KGRIB=NGRB100U)
  ENDIF
  IF (YSD_ACT_VD%L_100V) THEN
    YSD_VD%Y100V=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y100V,KGRIB=NGRB100V)
  ENDIF
  IF (YSD_ACT_VD%L_ZUST) THEN
    YSD_VD%YZUST=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YZUST,KGRIB=NGRBZUST)
  ENDIF
  IF (YSD_ACT_VD%L_10NU) THEN
    YSD_VD%Y10NU=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10NU,KGRIB=NGRB10NU, CDNAME='10M NEUTRL UWIND',KTRAJ=1,KREQIN=IREQIN)
  ENDIF
  IF (YSD_ACT_VD%L_10NV) THEN
    YSD_VD%Y10NV=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10NV,KGRIB=NGRB10NV, CDNAME='10M NEUTRL VWIND',KTRAJ=1,KREQIN=IREQIN)
  ENDIF
  IF (YSD_ACT_VD%L_DNDZN) THEN
    YSD_VD%YDNDZN=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YDNDZN,KGRIB=NGRBDNDZN)
  ENDIF
  IF (YSD_ACT_VD%L_DNDZA) THEN
    YSD_VD%YDNDZA=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YDNDZA,KGRIB=NGRBDNDZA)
  ENDIF
  IF (YSD_ACT_VD%L_DCTB) THEN
    YSD_VD%YDCTB=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YDCTB,KGRIB=NGRBDCTB)
  ENDIF
  IF (YSD_ACT_VD%L_TPLB) THEN
    YSD_VD%YTPLB=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTPLB,KGRIB=NGRBTPLB)
  ENDIF
  IF (YSD_ACT_VD%L_TPLT) THEN
    YSD_VD%YTPLT=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTPLT,KGRIB=NGRBTPLT)
  ENDIF
  IF(YSD_ACT_VD%L_TCGHG) THEN
    YSD_VD%YTCGHG => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+NGHG-1)
    DO JGHG=1,NGHG
      IF(YGHG(JGHG)%LGP) THEN
        CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCGHG(JGHG),YGHG(JGHG)%IGRBCODE)
      ELSE
        YSD_VDD%IPTR = YSD_VDD%IPTR+1
      ENDIF
    ENDDO
  ENDIF
  IF(YSD_ACT_VD%L_TCGRG) THEN
    YSD_VD%YTCGRG => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+NGRG-1)
    DO JGRG=1,NGRG
      IF(YGRG(JGRG)%LGP) THEN
! nl ext
        CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCGRG(JGRG),YGRG(JGRG)%IGRIBTC)
      ELSE
        YSD_VDD%IPTR = YSD_VDD%IPTR+1
      ENDIF
    ENDDO
  ENDIF
  IF(YSD_ACT_VD%L_TCTRAC) THEN
    YSD_VD%YTCTRAC => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+NTRAC-1)
    DO JTRAC=1,NTRAC
      IF(YTRAC(JTRAC)%LGP) THEN
        CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YTCTRAC(JTRAC),YTRAC(JTRAC)%IGRBCODE)
      ELSE
        YSD_VDD%IPTR = YSD_VDD%IPTR+1
      ENDIF
    ENDDO
  ENDIF
  IF(YSD_ACT_VD%L_MX2T6) THEN
    YSD_VD%YMX2T6 => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+N6BINS-1)
    DO JBIN=1,N6BINS
      CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMX2T6(JBIN),KGRIB=NGRBMX2T6)
    ENDDO
  ENDIF
  IF(YSD_ACT_VD%L_MN2T6) THEN
    YSD_VD%YMN2T6 => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+N6BINS-1)
    DO JBIN=1,N6BINS
      CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YMN2T6(JBIN),KGRIB=NGRBMN2T6)
    ENDDO
  ENDIF
  IF(YSD_ACT_VD%L_10FG6) THEN
    YSD_VD%Y10FG6 => YSD_VD%YVD(YSD_VDD%IPTR:YSD_VDD%IPTR+N6BINS-1)
    DO JBIN=1,N6BINS
      CALL SETUP_SFLP2(YSD_VDD,YSD_VD%Y10FG6(JBIN),KGRIB=NGRB10FG6)
    ENDDO
  ENDIF
ELSE
  IF (YSD_ACT_VD%L_SUND) THEN
    YSD_VD%YSUND=> YSD_VD%YVD(YSD_VDD%IPTR)
    CALL SETUP_SFLP2(YSD_VDD,YSD_VD%YSUND,KGRIB=NGRBSUND,CDNAME='SUNSHI. DURATION',&
     & PDEFAULT=0.0_JPRB, KREQIN=-1)
  ENDIF
ENDIF


! * Group WS=WAVES (for IFS)

! - define which field must be active:  
IF(LECMWF) THEN
  YSD_ACT_WS%L_CHAR   =.TRUE.
  YSD_ACT_WS%L_USTOKES=.TRUE.
  YSD_ACT_WS%L_VSTOKES=.TRUE.
ELSE
  YSD_ACT_WS%L_CHAR   =.FALSE.
  YSD_ACT_WS%L_USTOKES=.FALSE.
  YSD_ACT_WS%L_VSTOKES=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVWV=0
IF (YSD_ACT_WS%L_CHAR)    IVWV=IVWV+1
IF (YSD_ACT_WS%L_USTOKES) IVWV=IVWV+1
IF (YSD_ACT_WS%L_VSTOKES) IVWV=IVWV+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_WS%YWS)
IF(ASSOCIATED(YSD_WS%YWS)) DEALLOCATE(YSD_WS%YWS)
ALLOCATE(YSD_WS%YWS(JPMAXSFLDS))
CALL INI_SFLP2(YSD_WSD,YSD_WS%YWS,IVWV,.FALSE.,'WAVES - SD_WS   ')
YSD_WS%YCHAR   => YSD_WS%YWS(JPMAXSFLDS)
YSD_WS%YUSTOKES => YSD_WS%YWS(JPMAXSFLDS)
YSD_WS%YVSTOKES => YSD_WS%YWS(JPMAXSFLDS)

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSD_ACT_WS%L_CHAR) THEN
    IREQIN=-1 ; IF(LWCOU.AND.LWCOU2W) IREQIN=1
    ITRAJ = 2 ; IF(LVDFTRAJ) ITRAJ = 1 ! PUT CHARNOCK IN traj0XX FILES
    YSD_WS%YCHAR => YSD_WS%YWS(YSD_WSD%IPTR)
    CALL SETUP_SFLP2(YSD_WSD,YSD_WS%YCHAR,KGRIB=NGRBCHNK,&
     & PDEFAULT=0.018_JPRB,KTRAJ=ITRAJ,KREQIN=IREQIN, CDNAME='CHARNOCK_PARAM  ')
  ENDIF
  IF (YSD_ACT_WS%L_USTOKES) THEN
    YSD_WS%YUSTOKES => YSD_WS%YWS(YSD_WSD%IPTR)
    CALL SETUP_SFLP2(YSD_WSD,YSD_WS%YUSTOKES,CDNAME='U-STOKES_DRIFT  ')
  ENDIF
  IF (YSD_ACT_WS%L_VSTOKES) THEN
    YSD_WS%YVSTOKES => YSD_WS%YWS(YSD_WSD%IPTR)
    CALL SETUP_SFLP2(YSD_WSD,YSD_WS%YVSTOKES,CDNAME='V-STOKES_DRIFT  ')
  ENDIF
ELSE
  ! no field currently active
ENDIF


! * Group WW=WAVES (for WAM)

! - define which field must be active:  
IF(LECMWF) THEN
  YSD_ACT_WW%L_U10N =.TRUE.
  YSD_ACT_WW%L_V10N =.TRUE.
  YSD_ACT_WW%L_RHO  =.TRUE.
  YSD_ACT_WW%L_ZIL  =.TRUE.
  YSD_ACT_WW%L_CIF  =.TRUE.
  YSD_ACT_WW%L_UCURW=LWCUR
  YSD_ACT_WW%L_VCURW=LWCUR
ELSE
  YSD_ACT_WW%L_U10N =.FALSE.
  YSD_ACT_WW%L_V10N =.FALSE.
  YSD_ACT_WW%L_RHO  =.FALSE.
  YSD_ACT_WW%L_ZIL  =.FALSE.
  YSD_ACT_WW%L_CIF  =.FALSE.
  YSD_ACT_WW%L_UCURW=.FALSE.
  YSD_ACT_WW%L_VCURW=.FALSE.
ENDIF

! - compute number of fields allocated in this group:
IVWW=0
IF (YSD_ACT_WW%L_U10N ) IVWW=IVWW+1
IF (YSD_ACT_WW%L_V10N ) IVWW=IVWW+1
IF (YSD_ACT_WW%L_RHO  ) IVWW=IVWW+1
IF (YSD_ACT_WW%L_ZIL  ) IVWW=IVWW+1
IF (YSD_ACT_WW%L_CIF  ) IVWW=IVWW+1
IF (YSD_ACT_WW%L_UCURW) IVWW=IVWW+1
IF (YSD_ACT_WW%L_VCURW) IVWW=IVWW+1

! - call to INI_SFLP2, allocation:
IF(LL_FIRST_CALL) NULLIFY(YSD_WW%YWW)
IF(ASSOCIATED(YSD_WW%YWW)) DEALLOCATE(YSD_WW%YWW)
ALLOCATE(YSD_WW%YWW(JPMAXSFLDS))
CALL INI_SFLP2(YSD_WWD,YSD_WW%YWW,IVWW,.FALSE.,'WAVES - SD_WW   ')
YSD_WW%YU10N   => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YV10N   => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YRHO    => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YZIL    => YSD_WW%YWW(JPMAXSFLDS)
YSD_WW%YCIF    => YSD_WW%YWW(JPMAXSFLDS)
IF (LWCUR) THEN
 YSD_WW%YUCURW => YSD_WW%YWW(JPMAXSFLDS)
 YSD_WW%YVCURW => YSD_WW%YWW(JPMAXSFLDS)
ENDIF

! - set-up attributes for active fields:
IF(LECMWF) THEN
  IF (YSD_ACT_WW%L_U10N) THEN
    YSD_WW%YU10N => YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YU10N, CDNAME='10m_NEUTRAL_U   ')
  ENDIF
  IF (YSD_ACT_WW%L_V10N) THEN
    YSD_WW%YV10N => YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YV10N, CDNAME='10m_NEUTRAL_V   ')
  ENDIF
  IF (YSD_ACT_WW%L_RHO) THEN
    YSD_WW%YRHO => YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YRHO,  CDNAME='SURFACE_DENSITY ')
  ENDIF
  IF (YSD_ACT_WW%L_ZIL) THEN
    YSD_WW%YZIL => YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YZIL,  CDNAME='Zi/L            ')
  ENDIF
  IF (YSD_ACT_WW%L_CIF) THEN
    YSD_WW%YCIF => YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YCIF,  CDNAME='SEA_ICE_FRACTION')
  ENDIF
  IF (YSD_ACT_WW%L_UCURW) THEN
    YSD_WW%YUCURW=> YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YUCURW, CDNAME='OCEAN U-CURRENT ')
  ENDIF
  IF (YSD_ACT_WW%L_VCURW) THEN
    YSD_WW%YVCURW=> YSD_WW%YWW(YSD_WWD%IPTR)
    CALL SETUP_SFLP2(YSD_WWD,YSD_WW%YVCURW, CDNAME='OCEAN V-CURRENT ')
  ENDIF
ELSE
  ! no field currently active
ENDIF


! * Group VX=VCLIX

! - define which field must be active:  
IF(LECMWF) THEN
  YSD_ACT_VX%L_ORO =.FALSE.
  YSD_ACT_VX%L_TSC =.FALSE.
  YSD_ACT_VX%L_PWS =.FALSE.
  YSD_ACT_VX%L_PWP =.FALSE.
  YSD_ACT_VX%L_SNO =.FALSE.
  YSD_ACT_VX%L_TPC =.FALSE.
  YSD_ACT_VX%L_SAB =.FALSE.
  YSD_ACT_VX%L_XD2 =.FALSE.
  YSD_ACT_VX%L_LSM =.FALSE.
  YSD_ACT_VX%L_IVEG=.FALSE.
ELSE
  YSD_ACT_VX%L_ORO =(LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX)
  YSD_ACT_VX%L_TSC =(LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX)
  YSD_ACT_VX%L_PWS =(LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX).AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_PWP =(LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX).AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_SNO =(LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX).AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_TPC =(LCANARI.OR.LLFP_CLASSIC.OR.LLFP_SURFEX).AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_SAB =(LLFP_CLASSIC.OR.LLFP_SURFEX).AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_XD2 =(LLFP_CLASSIC.OR.LLFP_SURFEX).AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_LSM =LLFP_SURFEX.AND.(.NOT.LLASSIMAROME)
  YSD_ACT_VX%L_IVEG=LLFP_SURFEX.AND.(.NOT.LLASSIMAROME)
ENDIF

! - compute number of fields allocated in this group:
IVCLIX=0
IF (YSD_ACT_VX%L_ORO ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_TSC ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_PWS ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_PWP ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_SNO ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_TPC ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_SAB ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_XD2 ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_LSM ) IVCLIX=IVCLIX+1
IF (YSD_ACT_VX%L_IVEG) IVCLIX=IVCLIX+1

! - call to INI_SFLP2, allocation:
IF (LL_FIRST_CALL) NULLIFY(YSD_VX%YVX)
IF(ASSOCIATED(YSD_VX%YVX)) DEALLOCATE(YSD_VX%YVX)
ALLOCATE(YSD_VX%YVX(JPMAXSFLDS))
CALL INI_SFLP2(YSD_VXD,YSD_VX%YVX,IVCLIX,.FALSE.,'VCLIX - SD_VX   ')
YSD_VX%YORO   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YTSC   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YPWS   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YPWP   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YSNO   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YTPC   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YSAB   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YXD2   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YLSM   => YSD_VX%YVX(JPMAXSFLDS)
YSD_VX%YIVEG  => YSD_VX%YVX(JPMAXSFLDS)

! - set-up attributes for active fields:
IF (LECMWF) THEN
  ! no field currently active
ELSE
  IF (YSD_ACT_VX%L_ORO) THEN
    YSD_VX%YORO   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YORO, CDNAME='SURFGEOPOTENTIEL ')
  ENDIF
  IF (YSD_ACT_VX%L_TSC) THEN
    YSD_VX%YTSC   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YTSC, CDNAME='SURFTEMPERATURE  ')
  ENDIF
  IF (YSD_ACT_VX%L_PWS) THEN
    YSD_VX%YPWS   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YPWS, CDNAME='SURFPROP.RMAX.EAU')
  ENDIF
  IF (YSD_ACT_VX%L_PWP) THEN
    YSD_VX%YPWP   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YPWP, CDNAME='PROFPROP.RMAX.EAU')
  ENDIF
  IF (YSD_ACT_VX%L_SNO) THEN
    YSD_VX%YSNO   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YSNO ,CDNAME='SURFRESERV.NEIGE ')
  ENDIF
  IF (YSD_ACT_VX%L_TPC) THEN
    YSD_VX%YTPC   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YTPC ,CDNAME='PROFTEMPERATURE  ')
  ENDIF
  IF (YSD_ACT_VX%L_SAB) THEN
    YSD_VX%YSAB   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YSAB ,CDNAME='SURFPROP.SABLE   ')
  ENDIF
  IF (YSD_ACT_VX%L_XD2) THEN
    YSD_VX%YXD2   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YXD2 ,CDNAME='SURFEPAIS.SOL    ')
  ENDIF
  IF (YSD_ACT_VX%L_LSM) THEN
    YSD_VX%YLSM   => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YLSM ,CDNAME='SURFIND.TERREMER ')
  ENDIF
  IF (YSD_ACT_VX%L_IVEG) THEN
    YSD_VX%YIVEG  => YSD_VX%YVX(YSD_VXD%IPTR)
    CALL SETUP_SFLP2(YSD_VXD,YSD_VX%YIVEG,CDNAME='SURFIND.VEG.DOMI ')
  ENDIF
ENDIF

! * Group XA=VEXTRA

IF (LL_FIRST_CALL) NULLIFY(YSD_XA%YXA)
IF(ASSOCIATED(YSD_XA%YXA)) DEALLOCATE(YSD_XA%YXA)
ALLOCATE(YSD_XA%YXA(NVEXTR))
CALL INI_SFLP3(YSD_XAD,YSD_XA%YXA,NVEXTR,NCEXTR,.FALSE.,'VEXTRA - SD_XA  ')
IF(YSD_XAD%NUMFLDS > 0) THEN
  DO JFLD=1,YSD_XAD%NUMFLDS
    CALL SETUP_SFLP3(YSD_XAD,YSD_XA%YXA(JFLD),&
     & KGRIB   =SPREAD(NVEXTRAGB(JFLD),1,YSD_XAD%NLEVS),&
     & CDNAME  =SPREAD(CVEXTRA(JFLD)  ,1,YSD_XAD%NLEVS),&
     & PDEFAULT=SPREAD(VEXTRADF(JFLD) ,1,YSD_XAD%NLEVS))
  ENDDO
ENDIF

! * Group XR=VEXTRA-radiation

IF (LL_FIRST_CALL) NULLIFY(YSD_XR%YXA)
IF(ASSOCIATED(YSD_XR%YXA)) DEALLOCATE(YSD_XR%YXA)
ALLOCATE(YSD_XR%YXA(NVEXTRRAD))
IF (NVEXTRRAD > 0) THEN
  ICEXTR=NFLEVG
ELSE
  ICEXTR=0
ENDIF
CALL INI_SFLP3(YSD_XRD,YSD_XR%YXA,NVEXTRRAD,ICEXTR,.FALSE.,'VEXTRAD - SD_XR  ')
IF(YSD_XRD%NUMFLDS > 0) THEN
  DO JFLD=1,YSD_XRD%NUMFLDS
    CALL SETUP_SFLP3(YSD_XRD,YSD_XR%YXA(JFLD),&
     & KGRIB   =SPREAD(NVEXRADGB(JFLD),1,YSD_XRD%NLEVS),&
     & CDNAME  =SPREAD(CVEXRAD(JFLD)  ,1,YSD_XRD%NLEVS),&
     & PDEFAULT=SPREAD(VEXRADDF(JFLD) ,1,YSD_XRD%NLEVS))
  ENDDO
ENDIF

! * Group X2=VEXTR2

IF (LL_FIRST_CALL) NULLIFY(YSD_X2%YX2)
IF(ASSOCIATED(YSD_X2%YX2)) DEALLOCATE(YSD_X2%YX2)
ALLOCATE(YSD_X2%YX2(NVXTR2))
CALL INI_SFLP2(YSD_X2D,YSD_X2%YX2,NVXTR2,.FALSE.,'VEXTR2 - SD_X2  ')
IF(YSD_X2D%NUMFLDS > 0) THEN
  DO JFLD=1,YSD_X2D%NUMFLDS
    CALL SETUP_SFLP2(YSD_X2D,YSD_X2%YX2(JFLD),&
     & KGRIB   =NVEXTR2GB(JFLD),&
     & CDNAME  =CVEXTR2(JFLD)  ,&
     & PDEFAULT=VEXTR2DF(JFLD) )
  ENDDO
ENDIF

! * Group SFL=SFLUX

IF (LL_FIRST_CALL) NULLIFY(YSD_SFL%YSFL)
IF(ASSOCIATED(YSD_SFL%YSFL)) DEALLOCATE(YSD_SFL%YSFL)
ALLOCATE(YSD_SFL%YSFL(JPMAXSFLDS))
CALL INI_SFLP2(YSD_SFLD,YSD_SFL%YSFL,NSFLUX,.FALSE.,'SFLUX - SD_SFL  ')
IF (LEDKF) THEN
  DO JFLD=1,YSD_SFLD%NUMFLDS
    CALL SETUP_SFLP2(YSD_SFLD,YSD_SFL%YSFL(JFLD),CDNAME=CLSFLUX)
  ENDDO
ENDIF

! * Group SFO=SFORC

IF (LL_FIRST_CALL) NULLIFY(YSD_SFO%YSFO)
IF(ASSOCIATED(YSD_SFO%YSFO)) DEALLOCATE(YSD_SFO%YSFO)
ALLOCATE(YSD_SFO%YSFO(JPMAXSFLDS))
CALL INI_SFLP2(YSD_SFOD,YSD_SFO%YSFO,NSFORC,.FALSE.,'SFORC - SD_SFO  ')
IF (LSFORCS) THEN
  DO JFLD=1,YSD_SFOD%NUMFLDS
    WRITE(CLSFORC,FMT='(''SURFFORC'',I2.2)') JFLD
    CALL SETUP_SFLP2(YSD_SFOD,YSD_SFO%YSFO(JFLD),CDNAME=CLSFORC)
  ENDDO
ENDIF

!-------------------------------------------------------------------------

!*      3.  ADDITIONAL CHECKINGS AND PRINTINGS.
!           -----------------------------------

IF (LECMWF) THEN
ELSE
  IF ((LMPHYS.AND.LSOLV).AND.NCONF /= 923) THEN
    IF ( YSD_VVD%NUMFLDS < 7.OR.YSP_SBD%NLEVS < 1.OR.&
     & YSP_SBD%NUMFLDS < 2.OR.YSP_RRD%NUMFLDS < 3 ) THEN
      CALL ABOR1('SU_SURF_FLDS: LSOLV INCONSISTENT WITH YOMDPHY ')
    ENDIF
    IF (LFGEL.AND.(YSP_SBD%NUMFLDS < 3)) THEN
      CALL ABOR1('SU_SURF_FLDS: LFGEL INCONSISTENT WITH YOMDPHY ')
    ENDIF
    IF (LFGEL.AND.(YSP_RRD%NUMFLDS < 4)) THEN
      CALL ABOR1('SU_SURF_FLDS: LFGEL INCONSISTENT WITH YSP_RRD%NUMFLDS   ')
    ENDIF
    IF (LSNV) THEN
      IF (YSP_SGD%NUMFLDS < 3.OR.YSD_VFD%NUMFLDS < 10) THEN
        CALL ABOR1('SU_SURF_FLDS: LSNV INCONSISTENT WITH YOMDPHY ')
      ENDIF
      IF(.NOT.LSOLV .OR. .NOT.LNEIGE ) THEN
        CALL ABOR1('SU_SURF_FLDS: LSNV INCONSISTENT WITH LSOLV OR LNEIGE')
      ENDIF
    ENDIF
    IF (LVGSN.AND.(YSD_VVD%NUMFLDS < 10)) THEN
      CALL ABOR1('SU_SURF_FLDS: LVGSN INCONSISTENT WITH YSD_VVD%NUMFLDS  ')
    ENDIF
  ENDIF
ENDIF

WRITE(NULOUT,*) 'NSURF=',NSURF,' NSURFL=',NSURFL,' NDIMSURF=',NDIMSURF,&
 & ' NDIMSURFL=',NDIMSURFL
NGP5 = NOFFTRAJ
NTRAJ_CST = NOFFTRAJ_CST
WRITE(NULOUT,*) 'NGP5=',NGP5,' NTRAJ_CST=',NTRAJ_CST
WRITE(NULOUT,'(A,(10I4))') ' NSTRAJGRIB ',NSTRAJGRIB(1:NGP5)

LL_FIRST_CALL=.FALSE.

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU_SURF_FLDS',1,ZHOOK_HANDLE)

END SUBROUTINE SU_SURF_FLDS


